@@ -1,4 +1,4 @@
-/* $XConsortium: bdfread.c,v 1.22 94/04/17 20:17:10 gildea Exp $ */
+/* $Xorg: bdfread.c,v 1.3 2000/08/17 19:46:34 cpqbld Exp $ */
 
 /************************************************************************
 Copyright 1989 by Digital Equipment Corporation, Maynard, Massachusetts.
@@ -25,15 +25,9 @@ SOFTWARE.
 
 /*
 
-Copyright (c) 1994  X Consortium
+Copyright 1994, 1998  The Open Group
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+All Rights Reserved.
 
 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.
@@ -41,20 +35,24 @@ in all copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR
+IN NO EVENT SHALL THE OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR
 OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 
-Except as contained in this notice, the name of the X Consortium shall
+Except as contained in this notice, the name of The Open Group shall
 not be used in advertising or otherwise to promote the sale, use or
 other dealings in this Software without prior written authorization
-from the X Consortium.
+from The Open Group.
 
 */
+/* $XFree86: xc/lib/font/bitmap/bdfread.c,v 1.8 1999/08/01 07:56:53 dawes Exp $ */
 
+#ifndef FONTMODULE
 #include <ctype.h>
+#endif
 #include "fntfilst.h"
+#include "fontutil.h"
 /* use bitmap structure */
 #include "bitmap.h"
 #include "bdfint.h"
@@ -63,23 +61,14 @@ from the X Consortium.
 #define MAXENCODING 0xFFFF
 #define BDFLINELEN  1024
 
-extern int  bitmapGetGlyphs(), bitmapGetMetrics();
-extern int  bitmapGetBitmaps(), bitmapGetExtents();
-void        bdfUnloadFont();
-static Bool bdfPadToTerminal();
+static Bool bdfPadToTerminal(FontPtr pFont);
 extern int  bdfFileLineNum;
 
 /***====================================================================***/
 
 static Bool
-bdfReadBitmap(pCI, file, bit, byte, glyph, scan, sizes)
-    CharInfoPtr pCI;
-    FontFilePtr file;
-    int         bit,
-                byte,
-                glyph,
-                scan;
-    int         sizes[GLYPHPADOPTIONS];
+bdfReadBitmap(CharInfoPtr pCI, FontFilePtr file, int bit, int byte, 
+	      int glyph, int scan, CARD32 *sizes)
 {
     int         widthBits,
                 widthBytes,
@@ -93,7 +82,7 @@ bdfReadBitmap(pCI, file, bit, byte, glyph, scan, sizes)
     unsigned char *pInBits,
                *picture,
                *line = NULL;
-    char        lineBuf[BDFLINELEN];
+    unsigned char        lineBuf[BDFLINELEN];
 
     widthBits = GLYPHWIDTHPIXELS(pCI);
     height = GLYPHHEIGHTPIXELS(pCI);
@@ -199,13 +188,11 @@ BAILOUT:
 /***====================================================================***/
 
 static Bool
-bdfSkipBitmap(file, height)
-    FontFilePtr file;
-    int         height;
+bdfSkipBitmap(FontFilePtr file, int height)
 {
     unsigned char *line;
     int         i = 0;
-    char        lineBuf[BDFLINELEN];
+    unsigned char        lineBuf[BDFLINELEN];
 
     do {
 	line = bdfGetLine(file, lineBuf, BDFLINELEN);
@@ -221,16 +208,40 @@ bdfSkipBitmap(file, height)
 
 /***====================================================================***/
 
+static void
+bdfFreeFontBits(FontPtr pFont)
+{
+    BitmapFontPtr  bitmapFont;
+    BitmapExtraPtr bitmapExtra;
+    int         i, nencoding;
+
+    bitmapFont = (BitmapFontPtr) pFont->fontPrivate;
+    bitmapExtra = (BitmapExtraPtr) bitmapFont->bitmapExtra;
+    xfree(bitmapFont->ink_metrics);
+    if(bitmapFont->encoding) {
+        nencoding = (pFont->info.lastCol - pFont->info.firstCol + 1) *
+	    (pFont->info.lastRow - pFont->info.firstRow + 1);
+        for(i=0; i<NUM_SEGMENTS(nencoding); i++)
+            xfree(bitmapFont->encoding[i]);
+    }
+    xfree(bitmapFont->encoding);
+    for (i = 0; i < bitmapFont->num_chars; i++)
+	xfree(bitmapFont->metrics[i].bits);
+    xfree(bitmapFont->metrics);
+    if (bitmapExtra)
+    {
+	xfree (bitmapExtra->glyphNames);
+	xfree (bitmapExtra->sWidths);
+	xfree (bitmapExtra);
+    }
+    xfree(pFont->info.props);
+    xfree(bitmapFont);
+}
+
 
 static Bool
-bdfReadCharacters(file, pFont, pState, bit, byte, glyph, scan)
-    FontFilePtr file;
-    FontPtr     pFont;
-    bdfFileState *pState;
-    int         bit,
-                byte,
-                glyph,
-                scan;
+bdfReadCharacters(FontFilePtr file, FontPtr pFont, bdfFileState *pState, 
+		  int bit, int byte, int glyph, int scan)
 {
     unsigned char *line;
     register CharInfoPtr ci;
@@ -244,7 +255,7 @@ bdfReadCharacters(file, pFont, pState, bit, byte, glyph, scan)
     BitmapFontPtr  bitmapFont;
     BitmapExtraPtr bitmapExtra;
     CARD32     *bitmapsSizes;
-    char        lineBuf[BDFLINELEN];
+    unsigned char        lineBuf[BDFLINELEN];
     int         nencoding;
 
     bitmapFont = (BitmapFontPtr) pFont->fontPrivate;
@@ -277,6 +288,7 @@ bdfReadCharacters(file, pFont, pState, bit, byte, glyph, scan)
 		 sizeof(CharInfoRec));
 	goto BAILOUT;
     }
+    bzero((char *)ci, nchars * sizeof(CharInfoRec));
     bitmapFont->metrics = ci;
 
     if (bitmapExtra) {
@@ -327,7 +339,7 @@ bdfReadCharacters(file, pFont, pState, bit, byte, glyph, scan)
 	    bdfError("bad 'ENCODING' in BDF file\n");
 	    goto BAILOUT;
 	}
-	if (enc < -1 || t == 2 && enc2 < -1) {
+	if (enc < -1 || (t == 2 && enc2 < -1)) {
 	    bdfError("bad ENCODING value");
 	    goto BAILOUT;
 	}
@@ -410,7 +422,7 @@ bdfReadCharacters(file, pFont, pState, bit, byte, glyph, scan)
 		    (*p == ' ') || (*p == '\t');
 		    p++)
 		 /* empty for loop */ ;
-	    ci->metrics.attributes = bdfHexByte(p) << 8 + bdfHexByte(p + 2);
+	    ci->metrics.attributes = (bdfHexByte(p) << 8) + bdfHexByte(p + 2);
 	    line = bdfGetLine(file, lineBuf, BDFLINELEN);
 	} else
 	    ci->metrics.attributes = 0;
@@ -464,11 +476,13 @@ bdfReadCharacters(file, pFont, pState, bit, byte, glyph, scan)
 
     nencoding = (pFont->info.lastRow - pFont->info.firstRow + 1) *
 	(pFont->info.lastCol - pFont->info.firstCol + 1);
-    bitmapFont->encoding = (CharInfoPtr *) xalloc(nencoding * sizeof(CharInfoPtr));
+    bitmapFont->encoding = 
+      (CharInfoPtr **) xcalloc(NUM_SEGMENTS(nencoding),
+                               sizeof(CharInfoPtr*));
     if (!bitmapFont->encoding) {
 	bdfError("Couldn't allocate ppCI (%d,%d)\n",
-		 nencoding,
-		 sizeof(CharInfoPtr));
+                 NUM_SEGMENTS(nencoding),
+                 sizeof(CharInfoPtr*));
 	goto BAILOUT;
     }
     pFont->info.allExist = TRUE;
@@ -478,19 +492,26 @@ bdfReadCharacters(file, pFont, pState, bit, byte, glyph, scan)
 	    char_row++) {
 	if (bdfEncoding[char_row] == (CharInfoPtr *) NULL) {
 	    pFont->info.allExist = FALSE;
-	    for (char_col = pFont->info.firstCol;
-		    char_col <= pFont->info.lastCol;
-		    char_col++) {
-		bitmapFont->encoding[i++] = NullCharInfo;
-	    }
+            i += pFont->info.lastCol - pFont->info.firstCol + 1;
 	} else {
 	    for (char_col = pFont->info.firstCol;
 		    char_col <= pFont->info.lastCol;
 		    char_col++) {
 		if (!bdfEncoding[char_row][char_col])
 		    pFont->info.allExist = FALSE;
-		bitmapFont->encoding[i++] = bdfEncoding[char_row][char_col];
-	    }
+                else {
+                    if (!bitmapFont->encoding[SEGMENT_MAJOR(i)]) {
+                        bitmapFont->encoding[SEGMENT_MAJOR(i)]=
+                            (CharInfoPtr*)xcalloc(BITMAP_FONT_SEGMENT_SIZE,
+                                                  sizeof(CharInfoPtr));
+                        if (!bitmapFont->encoding[SEGMENT_MAJOR(i)])
+                            goto BAILOUT;
+                    }
+                    ACCESSENCODINGL(bitmapFont->encoding,i) = 
+                        bdfEncoding[char_row][char_col];
+                }
+                i++;
+            }
 	}
     }
     for (i = 0; i < 256; i++)
@@ -508,13 +529,11 @@ BAILOUT:
 /***====================================================================***/
 
 static Bool
-bdfReadHeader(file, pState)
-    FontFilePtr file;
-    bdfFileState *pState;
+bdfReadHeader(FontFilePtr file, bdfFileState *pState)
 {
     unsigned char *line;
     char        namebuf[BDFLINELEN];
-    char        lineBuf[BDFLINELEN];
+    unsigned char        lineBuf[BDFLINELEN];
 
     line = bdfGetLine(file, lineBuf, BDFLINELEN);
     if (!line || sscanf((char *) line, "STARTFONT %s", namebuf) != 1 ||
@@ -553,10 +572,7 @@ bdfReadHeader(file, pState)
 /***====================================================================***/
 
 static Bool
-bdfReadProperties(file, pFont, pState)
-    FontFilePtr file;
-    FontPtr     pFont;
-    bdfFileState *pState;
+bdfReadProperties(FontFilePtr file, FontPtr pFont, bdfFileState *pState)
 {
     int         nProps, props_left,
                 nextProp;
@@ -566,12 +582,12 @@ bdfReadProperties(file, pFont, pState)
                 secondbuf[BDFLINELEN],
                 thirdbuf[BDFLINELEN];
     unsigned char *line;
-    char        lineBuf[BDFLINELEN];
+    unsigned char        lineBuf[BDFLINELEN];
     BitmapFontPtr  bitmapFont = (BitmapFontPtr) pFont->fontPrivate;
 
     line = bdfGetLine(file, lineBuf, BDFLINELEN);
     if (!line || !bdfIsPrefix(line, "STARTPROPERTIES")) {
-	bdfError(file, "missing 'STARTPROPERTIES'\n");
+	bdfError("missing 'STARTPROPERTIES'\n");
 	return (FALSE);
     }
     if (sscanf((char *) line, "STARTPROPERTIES %d", &nProps) != 1) {
@@ -595,6 +611,8 @@ bdfReadProperties(file, pFont, pState)
 						      sizeof(FontPropRec));
 	goto BAILOUT;
     }
+    bzero((char *)props, (nProps + BDF_GENPROPS) * sizeof(FontPropRec));
+
     nextProp = 0;
     props_left = nProps;
     while (props_left-- > 0) {
@@ -620,7 +638,7 @@ bdfReadProperties(file, pFont, pState)
 	    if (secondbuf[0] == '"') {
 		stringProps[nextProp] = TRUE;
 		props[nextProp].value =
-		    bdfGetPropertyValue(line + strlen(namebuf) + 1);
+		    bdfGetPropertyValue((char *)line + strlen(namebuf) + 1);
 		if (!props[nextProp].value)
 		    goto BAILOUT;
 		break;
@@ -641,7 +659,7 @@ bdfReadProperties(file, pFont, pState)
 	    if (secondbuf[0] == '"') {
 		stringProps[nextProp] = TRUE;
 		props[nextProp].value =
-		    bdfGetPropertyValue(line + strlen(namebuf) + 1);
+		    bdfGetPropertyValue((char *)line + strlen(namebuf) + 1);
 		if (!props[nextProp].value)
 		    goto BAILOUT;
 		break;
@@ -748,14 +766,16 @@ BAILOUT:
 
 /***====================================================================***/
 
+static void
+bdfUnloadFont(FontPtr pFont)
+{
+    bdfFreeFontBits (pFont);
+    DestroyFontRec(pFont);
+}
+
 int
-bdfReadFont(pFont, file, bit, byte, glyph, scan)
-    FontPtr     pFont;
-    FontFilePtr file;
-    int         bit,
-                byte,
-                glyph,
-                scan;
+bdfReadFont(FontPtr pFont, FontFilePtr file, 
+	    int bit, int byte, int glyph, int scan)
 {
     bdfFileState state;
     xCharInfo  *min,
@@ -773,6 +793,7 @@ bdfReadFont(pFont, file, bit, byte, glyph, scan)
     bitmapFont = (BitmapFontPtr) xalloc(sizeof(BitmapFontRec));
     if (!bitmapFont)
 	goto BAILOUT;
+    bzero((char *)bitmapFont, sizeof(BitmapFontRec));
 
     pFont->fontPrivate = (pointer) bitmapFont;
     bitmapFont->metrics = 0;
@@ -780,7 +801,12 @@ bdfReadFont(pFont, file, bit, byte, glyph, scan)
     bitmapFont->bitmaps = 0;
     bitmapFont->encoding = 0;
     bitmapFont->pDefault = NULL;
+
     bitmapFont->bitmapExtra = (BitmapExtraPtr) xalloc(sizeof(BitmapExtraRec));
+    if (!bitmapFont->bitmapExtra)
+        goto BAILOUT;
+    bzero((char *)bitmapFont->bitmapExtra, sizeof(BitmapExtraRec));
+    
     bitmapFont->bitmapExtra->glyphNames = 0;
     bitmapFont->bitmapExtra->sWidths = 0;
 
@@ -800,7 +826,8 @@ bdfReadFont(pFont, file, bit, byte, glyph, scan)
 	    cols = pFont->info.lastCol - pFont->info.firstCol + 1;
 	    r = r - pFont->info.firstRow;
 	    c = c - pFont->info.firstCol;
-	    bitmapFont->pDefault = bitmapFont->encoding[r * cols + c];
+	    bitmapFont->pDefault = ACCESSENCODING(bitmapFont->encoding, 
+                                                 r * cols + c);
 	}
     }
     pFont->bit = bit;
@@ -851,38 +878,14 @@ BAILOUT:
     return AllocError;
 }
 
-bdfFreeFontBits(pFont)
-    FontPtr pFont;
-{
-    BitmapFontPtr  bitmapFont;
-    BitmapExtraPtr bitmapExtra;
-    int         i;
-
-    bitmapFont = (BitmapFontPtr) pFont->fontPrivate;
-    bitmapExtra = (BitmapExtraPtr) bitmapFont->bitmapExtra;
-    xfree(bitmapFont->ink_metrics);
-    xfree(bitmapFont->encoding);
-    for (i = 0; i < bitmapFont->num_chars; i++)
-	xfree(bitmapFont->metrics[i].bits);
-    xfree(bitmapFont->metrics);
-    if (bitmapExtra)
-    {
-	xfree (bitmapExtra->glyphNames);
-	xfree (bitmapExtra->sWidths);
-	xfree (bitmapExtra);
-    }
-    xfree(pFont->info.props);
-    xfree(bitmapFont);
-}
-
 int
-bdfReadFontInfo(pFontInfo, file)
-    FontInfoPtr pFontInfo;
-    FontFilePtr file;
+bdfReadFontInfo(FontInfoPtr pFontInfo, FontFilePtr file)
 {
     FontRec     font;
     int         ret;
 
+    bzero(&font, sizeof (FontRec));
+
     ret = bdfReadFont(&font, file, MSBFirst, LSBFirst, 1, 1);
     if (ret == Successful) {
 	*pFontInfo = font.info;
@@ -894,18 +897,8 @@ bdfReadFontInfo(pFontInfo, file)
     return ret;
 }
 
-void
-bdfUnloadFont(pFont)
-    FontPtr     pFont;
-{
-    bdfFreeFontBits (pFont);
-    xfree (pFont->devPrivates);
-    xfree(pFont);
-}
-
 static Bool
-bdfPadToTerminal(pFont)
-    FontPtr     pFont;
+bdfPadToTerminal(FontPtr pFont)
 {
     BitmapFontPtr  bitmapFont;
     BitmapExtraPtr bitmapExtra;
@@ -916,12 +909,15 @@ bdfPadToTerminal(pFont)
                 h;
 
     bitmapFont = (BitmapFontPtr) pFont->fontPrivate;
+
+    bzero(&new, sizeof(CharInfoRec)); 
     new.metrics.ascent = pFont->info.fontAscent;
     new.metrics.descent = pFont->info.fontDescent;
     new.metrics.leftSideBearing = 0;
     new.metrics.rightSideBearing = pFont->info.minbounds.characterWidth;
     new.metrics.characterWidth = new.metrics.rightSideBearing;
     new_size = BYTES_FOR_GLYPH(&new, pFont->glyph);
+
     for (i = 0; i < bitmapFont->num_chars; i++) {
 	new.bits = (char *) xalloc(new_size);
 	if (!new.bits)
