@@ -1,4 +1,4 @@
-/* $TOG: PclGC.c /main/3 1997/06/11 17:53:43 samborn $ */
+/* $Xorg: PclGC.c,v 1.4 2000/08/17 19:48:08 cpqbld Exp $ */
 /*******************************************************************
 **
 **    *********************************************************
@@ -44,7 +44,7 @@ not be used in advertising or otherwise to promote the sale, use or other
 dealings in this Software without prior written authorization from said
 copyright holders.
 */
-/* $XFree86: xc/programs/Xserver/Xprint/pcl/PclGC.c,v 1.3.2.2 1998/10/04 13:36:44 hohndel Exp $ */
+/* $XFree86: xc/programs/Xserver/Xprint/pcl/PclGC.c,v 1.8 2001/01/17 22:36:30 dawes Exp $ */
 
 #include "gcstruct.h"
 
@@ -52,6 +52,8 @@ copyright holders.
 #include "pixmapstr.h"
 #include "colormapst.h"
 #include "windowstr.h"
+#include "cfb.h"
+#include "cfb32.h"
 #include "migc.h"
 #include "scrnintstr.h"
 #include "resource.h"
@@ -98,24 +100,20 @@ static GCFuncs PclGCFuncs =
 ;
 
 Bool
-PclCreateGC( pGC )
-     GCPtr pGC;
+PclCreateGC(GCPtr pGC)
 {
-    PclGCPrivPtr pPriv = pGC->devPrivates[PclGCPrivateIndex].ptr;
-
     if( pGC->depth == 1 )
       {
 	  if( mfbCreateGC( pGC ) == FALSE )
 	    return FALSE;
       }
-    else if( pGC->depth <= 8 )
-      {
-	  if( cfbCreateGC( pGC ) == FALSE )
-	    return FALSE;
-      }
     else if( pGC->depth <= 32 )
       {
+#if PSZ == 8 
+	  if( cfbCreateGC( pGC ) == FALSE )
+#else
 	  if( cfb32CreateGC( pGC ) == FALSE )
+#endif
 	    return FALSE;
       }
     else
@@ -127,33 +125,23 @@ PclCreateGC( pGC )
     pGC->ops = &PclGCOps;
     pGC->funcs = &PclGCFuncs;
 
-    pPriv->pCompositeClip = NULL;
-    pPriv->freeCompClip = FALSE;
-    
     return TRUE;
 }
 
 void
 PclDestroyGC(GCPtr pGC)
 {
-    PclGCPrivPtr pPriv = pGC->devPrivates[PclGCPrivateIndex].ptr;
-    extern int mfbGCPrivateIndex;
-    
     /* Handle the mfb and cfb, which share a GC private struct */
-    miRegisterGCPrivateIndex( mfbGCPrivateIndex );
     miDestroyGC( pGC );
-    
-    if( pPriv->freeCompClip == TRUE )
-      REGION_DESTROY( pGC->pScreen, pPriv->pCompositeClip );
 }
 
 
 int
-PclGetDrawablePrivateStuff( pDrawable, gc, valid, file )
-     DrawablePtr pDrawable;
-     GC *gc;
-     unsigned long *valid;
-     FILE **file;
+PclGetDrawablePrivateStuff(
+     DrawablePtr pDrawable,
+     GC *gc,
+     unsigned long *valid,
+     FILE **file)
 {
     XpContextPtr pCon;
     PclPixmapPrivPtr pPriv;
@@ -187,9 +175,9 @@ PclGetDrawablePrivateStuff( pDrawable, gc, valid, file )
 }
 
 void
-PclSetDrawablePrivateGC( pDrawable, gc )
-     DrawablePtr pDrawable;
-     GC gc;
+PclSetDrawablePrivateGC(
+     DrawablePtr pDrawable,
+     GC gc)
 {
     PixmapPtr pix;
     XpContextPtr pCon;
@@ -334,10 +322,10 @@ PclSendPattern(char *bits,
 }
 
 int
-PclUpdateDrawableGC( pGC, pDrawable, outFile )
-     GCPtr pGC;
-     DrawablePtr pDrawable;
-     FILE **outFile;
+PclUpdateDrawableGC(
+     GCPtr pGC,
+     DrawablePtr pDrawable,
+     FILE **outFile)
 {
     Mask drawableMask, changeMask = 0;
     GC dGC;
@@ -405,12 +393,14 @@ PclUpdateDrawableGC( pGC, pDrawable, outFile )
 	    changeMask |= GCTileStipYOrigin;
 	  
 	  if( dGC.numInDashList == pGC->numInDashList )
+	  {
 	    for( i = 0; i < dGC.numInDashList; i++ )
 	      if( cPriv->dash[i] != pGC->dash[i] )
 		{
 		    changeMask |= GCDashList;
 		    break;
 		}
+	  }
 	  else
 	    changeMask |= GCDashList;
       }
@@ -540,7 +530,7 @@ PclUpdateDrawableGC( pGC, pDrawable, outFile )
 	    }
 	  else /* PseudoColor or StaticGray */
 	    {
-		sprintf( t, "SP%d;", pGC->fgPixel );
+		sprintf( t, "SP%ld;", (long) pGC->fgPixel );
 		SEND_PCL( *outFile, t );
 	    }
 #else
@@ -668,7 +658,7 @@ PclUpdateDrawableGC( pGC, pDrawable, outFile )
 		sz = h * BitmapBytePad( w );
 
 		bits = (char *)xalloc( sz );
-		mfbGetImage(pGC->tile.pixmap, 0, 0, w, h, XYPixmap, ~0, bits);
+		mfbGetImage(&(pGC->tile.pixmap->drawable), 0, 0, w, h, XYPixmap, ~0, bits);
 		PclSendPattern( bits, sz, 1, h, w, 100, *outFile );
 		xfree( bits );
 	    }
@@ -676,19 +666,21 @@ PclUpdateDrawableGC( pGC, pDrawable, outFile )
 	    {
 		sz = h * PixmapBytePad( w, 8 );
 		bits = (char *)xalloc( sz );
-		cfbGetImage(pGC->tile.pixmap, 0, 0, w, h, ZPixmap, ~0, bits);
+		cfbGetImage(&(pGC->tile.pixmap->drawable), 0, 0, w, h, ZPixmap, ~0, bits);
 		PclSendPattern( bits, sz, 8, h, w, 100, *outFile );
 		xfree( bits );
 	    }
+#if PSZ == 32
 	  else
 	    {
 		sz = h * PixmapBytePad( w, 24 );
 		
 		bits = (char *)xalloc( sz );
-		cfb32GetImage(pGC->tile.pixmap, 0, 0, w, h, ZPixmap, ~0, bits);
+		cfb32GetImage(&(pGC->tile.pixmap->drawable), 0, 0, w, h, ZPixmap, ~0, bits);
 		PclSendPattern( bits, sz, 24, h, w, 100, *outFile );
 		xfree( bits );
 	    }
+#endif
       }
 
     if( changeMask & ( GCTileStipXOrigin | GCTileStipYOrigin ) )
@@ -723,7 +715,7 @@ PclUpdateDrawableGC( pGC, pDrawable, outFile )
 		sz = h * BitmapBytePad( w );
 
 		bits = (char *)xalloc( sz );
-		mfbGetImage( pGC->stipple, 0, 0, w, h, XYPixmap, ~0, bits );
+		mfbGetImage( &(pGC->stipple->drawable), 0, 0, w, h, XYPixmap, ~0, bits );
 
 		w2 = ( w / 8 ) + ( ( w%8 ) ? 1 : 0 );
 		/*
@@ -766,31 +758,31 @@ PclUpdateDrawableGC( pGC, pDrawable, outFile )
 		  {
 		      mfbValidateGC( scratchGC, ~0L,
 				    (DrawablePtr)scratchPix );
-		      mfbCopyPlane( pGC->stipple,
+		      mfbCopyPlane( &(pGC->stipple->drawable),
 				   (DrawablePtr)scratchPix, scratchGC, 0,
 				   0, w, h, 0, 0, 1 );
-		      mfbGetImage( scratchPix, 0, 0, w, h, XYPixmap, ~0,
-				  bits );
-		  }
-		else if( pGC->depth <= 8 )
-		  {
-		      cfbValidateGC( scratchGC, ~0L,
-				    (DrawablePtr)scratchPix );
-		      cfbCopyPlane( pGC->stipple,
-				   (DrawablePtr)scratchPix, scratchGC, 0,
-				   0, w, h, 0, 0, 1 );
-		      cfbGetImage( scratchPix, 0, 0, w, h, ZPixmap, ~0, 
+		      mfbGetImage( &(scratchPix->drawable), 0, 0, w, h, XYPixmap, ~0,
 				  bits );
 		  }
 		else if( pGC->depth <= 32 )
 		  {
-		      cfb32ValidateGC( scratchGC, ~0L,
+#if PSZ == 8
+		      cfbValidateGC( scratchGC, ~0L,
 				    (DrawablePtr)scratchPix );
-		      cfb32CopyPlane( pGC->stipple,
+		      cfbCopyPlane( &(pGC->stipple->drawable),
 				   (DrawablePtr)scratchPix, scratchGC, 0,
 				   0, w, h, 0, 0, 1 );
-		      cfb32GetImage( scratchPix, 0, 0, w, h, ZPixmap, ~0, 
+		      cfbGetImage( &(scratchPix->drawable), 0, 0, w, h, ZPixmap, ~0, 
 				  bits );
+#else
+		      cfb32ValidateGC( scratchGC, ~0L,
+				      (DrawablePtr)scratchPix );
+		      cfb32CopyPlane( pGC->stipple,
+				     (DrawablePtr)scratchPix, scratchGC, 0,
+				     0, w, h, 0, 0, 1 );
+		      cfb32GetImage( scratchPix, 0, 0, w, h, ZPixmap, ~0,
+				    bits );
+#endif
 		  }
 		PclSendPattern( bits, sz, pGC->depth, h, w, 101, *outFile );
 		FreeScratchGC( scratchGC );
@@ -868,13 +860,11 @@ PclUpdateDrawableGC( pGC, pDrawable, outFile )
 
 
 void
-PclComputeCompositeClip(pGC, pDrawable)
-    GCPtr           pGC;
-    DrawablePtr     pDrawable;
+PclComputeCompositeClip(
+    GCPtr           pGC,
+    DrawablePtr     pDrawable)
 {
     ScreenPtr       pScreen = pGC->pScreen;
-    PclGCPrivPtr devPriv = (PclGCPrivPtr)
-      (pGC->devPrivates[PclGCPrivateIndex].ptr);
 
     if (pDrawable->type == DRAWABLE_WINDOW)
     {
@@ -892,7 +882,7 @@ PclComputeCompositeClip(pGC, pDrawable)
 	    pregWin = &pWin->clipList;
 	    freeTmpClip = FALSE;
 	}
-	freeCompClip = devPriv->freeCompClip;
+	freeCompClip = pGC->freeCompClip;
 
 	/*
 	 * if there is no client clip, we can get by with just keeping the
@@ -904,9 +894,9 @@ PclComputeCompositeClip(pGC, pDrawable)
 	if (pGC->clientClipType == CT_NONE)
 	{
 	    if (freeCompClip)
-		REGION_DESTROY(pScreen, devPriv->pCompositeClip);
-	    devPriv->pCompositeClip = pregWin;
-	    devPriv->freeCompClip = freeTmpClip;
+		REGION_DESTROY(pScreen, pGC->pCompositeClip);
+	    pGC->pCompositeClip = pregWin;
+	    pGC->freeCompClip = freeTmpClip;
 	}
 	else
 	{
@@ -925,7 +915,7 @@ PclComputeCompositeClip(pGC, pDrawable)
 
 	    if (freeCompClip)
 	    {
-		REGION_INTERSECT(pGC->pScreen, devPriv->pCompositeClip,
+		REGION_INTERSECT(pGC->pScreen, pGC->pCompositeClip,
 					    pregWin, pGC->clientClip);
 		if (freeTmpClip)
 		    REGION_DESTROY(pScreen, pregWin);
@@ -933,15 +923,15 @@ PclComputeCompositeClip(pGC, pDrawable)
 	    else if (freeTmpClip)
 	    {
 		REGION_INTERSECT(pScreen, pregWin, pregWin, pGC->clientClip);
-		devPriv->pCompositeClip = pregWin;
+		pGC->pCompositeClip = pregWin;
 	    }
 	    else
 	    {
-		devPriv->pCompositeClip = REGION_CREATE(pScreen, NullBox, 0);
-		REGION_INTERSECT(pScreen, devPriv->pCompositeClip,
+		pGC->pCompositeClip = REGION_CREATE(pScreen, NullBox, 0);
+		REGION_INTERSECT(pScreen, pGC->pCompositeClip,
 				       pregWin, pGC->clientClip);
 	    }
-	    devPriv->freeCompClip = TRUE;
+	    pGC->freeCompClip = TRUE;
 	    REGION_TRANSLATE(pScreen, pGC->clientClip,
 					 -(pDrawable->x + pGC->clipOrg.x),
 					 -(pDrawable->y + pGC->clipOrg.y));
@@ -957,23 +947,23 @@ PclComputeCompositeClip(pGC, pDrawable)
 	pixbounds.x2 = pDrawable->width;
 	pixbounds.y2 = pDrawable->height;
 
-	if (devPriv->freeCompClip)
+	if (pGC->freeCompClip)
 	{
-	    REGION_RESET(pScreen, devPriv->pCompositeClip, &pixbounds);
+	    REGION_RESET(pScreen, pGC->pCompositeClip, &pixbounds);
 	}
 	else
 	{
-	    devPriv->freeCompClip = TRUE;
-	    devPriv->pCompositeClip = REGION_CREATE(pScreen, &pixbounds, 1);
+	    pGC->freeCompClip = TRUE;
+	    pGC->pCompositeClip = REGION_CREATE(pScreen, &pixbounds, 1);
 	}
 
 	if (pGC->clientClipType == CT_REGION)
 	{
-	    REGION_TRANSLATE(pScreen, devPriv->pCompositeClip,
+	    REGION_TRANSLATE(pScreen, pGC->pCompositeClip,
 					 -pGC->clipOrg.x, -pGC->clipOrg.y);
-	    REGION_INTERSECT(pScreen, devPriv->pCompositeClip,
-				devPriv->pCompositeClip, pGC->clientClip);
-	    REGION_TRANSLATE(pScreen, devPriv->pCompositeClip,
+	    REGION_INTERSECT(pScreen, pGC->pCompositeClip,
+				pGC->pCompositeClip, pGC->clientClip);
+	    REGION_TRANSLATE(pScreen, pGC->pCompositeClip,
 					 pGC->clipOrg.x, pGC->clipOrg.y);
 	}
     }	/* end of composite clip for pixmap */
@@ -992,16 +982,11 @@ PclComputeCompositeClip(pGC, pDrawable)
 
 /*ARGSUSED*/
 void
-PclValidateGC( pGC, changes, pDrawable )
-     GCPtr pGC;
-     unsigned long changes;
-     DrawablePtr pDrawable;
+PclValidateGC(
+     GCPtr pGC,
+     unsigned long changes,
+     DrawablePtr pDrawable)
 {
-    XpContextPtr pCon;
-    PclContextPrivPtr pConPriv;
-    extern int mfbGCPrivateIndex;
-    extern int cfbGCPrivateIndex;
-    
     /*
      * Pixmaps should be handled by their respective validation
      * functions.
@@ -1010,18 +995,15 @@ PclValidateGC( pGC, changes, pDrawable )
       {
 	  if( pDrawable->depth == 1 )
 	    {
-		miRegisterGCPrivateIndex( mfbGCPrivateIndex );
 		mfbValidateGC( pGC, ~0, pDrawable );
 	    }
-	  else if( pDrawable->depth <= 8 )
-	    {
-		miRegisterGCPrivateIndex( cfbGCPrivateIndex );
-		cfbValidateGC( pGC, ~0, pDrawable );
-	    }
 	  else if( pDrawable->depth <= 32 )
 	    {
-		miRegisterGCPrivateIndex( cfbGCPrivateIndex );
+#if PSZ == 8
+		cfbValidateGC( pGC, ~0, pDrawable );
+#else
 		cfb32ValidateGC( pGC, ~0, pDrawable );
+#endif
 	    }
 	  return;
       }
@@ -1061,4 +1043,3 @@ PclValidateGC( pGC, changes, pDrawable )
     PclSetDrawablePrivateGC( pDrawable, *pGC, changes );
 */
 }
-
