@@ -1,16 +1,10 @@
-/* $TOG: record.c /main/8 1997/04/06 10:52:30 kaleb $ */
+/* $Xorg: record.c,v 1.3 2000/08/17 19:53:45 cpqbld Exp $ */
 
 /*
 
-Copyright (c) 1995  X Consortium
+Copyright 1995, 1998  The Open Group
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+All Rights Reserved.
 
 The above copyright notice and this permission notice shall be
 included in all copies or substantial portions of the Software.
@@ -18,32 +12,38 @@ included in all copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR
+IN NO EVENT SHALL THE OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR
 OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 
-Except as contained in this notice, the name of the X Consortium shall
+Except as contained in this notice, the name of The Open Group shall
 not be used in advertising or otherwise to promote the sale, use or
 other dealings in this Software without prior written authorization
-from the X Consortium.
+from The Open Group.
 
-Author: David P. Wiggins, X Consortium
+Author: David P. Wiggins, The Open Group
 
 This work benefited from earlier work done by Martha Zimet of NCD
 and Jim Haggerty of Metheus.
 
 */
+/* $XFree86: xc/programs/Xserver/record/record.c,v 1.6 1998/10/04 09:39:53 dawes Exp $ */
 
-#include <stdio.h>
-#include <assert.h>
 #define NEED_EVENTS
 #include "dixstruct.h"
 #include "extnsionst.h"
 #define _XRECORD_SERVER_
-#include "X11/extensions/recordstr.h"
+#include "recordstr.h"
 #include "set.h"
 
+#ifndef XFree86LOADER
+#include <stdio.h>
+#include <assert.h>
+#else
+#include "xf86_ansic.h"
+#endif
+
 static RESTYPE RTContext;   /* internal resource type for Record contexts */
 static int RecordErrorBase; /* first Record error number */
 
@@ -3021,3 +3021,4 @@ RecordExtensionInit()
     RecordErrorBase = extentry->errorBase;
 
 } /* RecordExtensionInit */
+
