@@ -1,15 +1,10 @@
-/* $XFree86: xc/programs/twm/menus.c,v 1.1.1.5.4.2 1998/10/24 08:48:23 hohndel Exp $ */
+/* $XFree86: xc/programs/twm/menus.c,v 1.8 2000/10/24 22:45:15 dawes Exp $ */
 /*****************************************************************************/
 /*
 
-Copyright (c) 1989  X Consortium
+Copyright 1989, 1998  The Open Group
 
-Permission is hereby granted, free of charge, to any person obtaining a copy
-of this software and associated documentation files (the "Software"), to deal
-in the Software without restriction, including without limitation the rights
-to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-copies of the Software, and to permit persons to whom the Software is
-furnished to do so, subject to the following conditions:
+All Rights Reserved.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -17,13 +12,13 @@ all copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
-X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-Except as contained in this notice, the name of the X Consortium shall not be
+Except as contained in this notice, the name of The Open Group shall not be
 used in advertising or otherwise to promote the sale, use or other dealings
-in this Software without prior written authorization from the X Consortium.
+in this Software without prior written authorization from The Open Group.
 
 */
 /**       Copyright 1988 by Evans & Sutherland Computer Corporation,        **/
@@ -54,7 +49,7 @@ in this Software without prior written authorization from the X Consortium.
 
 /***********************************************************************
  *
- * $XConsortium: menus.c /main/152 1996/02/02 14:26:16 kaleb $
+ * $Xorg: menus.c,v 1.5 2000/08/17 19:54:07 cpqbld Exp $
  *
  * twm menu code
  *
@@ -73,8 +68,12 @@ in this Software without prior written authorization from the X Consortium.
 #include "parse.h"
 #include "gram.h"
 #include "screen.h"
+#include "menus.h"
+#include "iconmgr.h"
+#include "add_window.h"
+#include "icons.h"
+#include "session.h"
 #include <X11/Xmu/CharSet.h>
-#include <X11/bitmaps/menu12>
 #include "version.h"
 #include <X11/extensions/sync.h>
 #include <X11/SM/SMlib.h>
@@ -109,19 +108,22 @@ static struct {
 } MenuOrigins[MAXMENUDEPTH];
 static Cursor LastCursor;
 
-void WarpAlongRing(), WarpToWindow();
 
 extern char *Action;
 extern int Context;
 extern TwmWindow *ButtonWindow, *Tmp_win;
 extern XEvent Event, ButtonEvent;
 extern char *InitFile;
-static void Identify();
+
+static Bool belongs_to_twm_window ( TwmWindow *t, Window w );
+static void Identify ( TwmWindow *t );
+static void send_clientmessage ( Window w, Atom a, Time timestamp );
 
 #define SHADOWWIDTH 5			/* in pixels */
 
 
 
+
 /***********************************************************************
  *
  *  Procedure:
@@ -374,7 +376,7 @@ void InitTitlebarButtons ()
 }
 
 
-
+void
 PaintEntry(mr, mi, exposure)
 MenuRoot *mr;
 MenuItem *mi;
@@ -401,9 +403,9 @@ int exposure;
 	    XFillRectangle(dpy, mr->w, Scr->NormalGC, 0, y_offset,
 		mr->width, Scr->EntryHeight);
 
-	    FBF(mi->hi_fore, mi->hi_back, Scr->MenuFont.font->fid);
+	    MyFont_ChangeGC(mi->hi_fore, mi->hi_back, &Scr->MenuFont);
 
-	    XDrawString(dpy, mr->w, Scr->NormalGC, mi->x,
+	    MyFont_DrawString(dpy, mr->w, &Scr->MenuFont, Scr->NormalGC, mi->x,
 		text_y, mi->item, mi->strlen);
 
 	    gc = Scr->NormalGC;
@@ -417,14 +419,15 @@ int exposure;
 		XFillRectangle(dpy, mr->w, Scr->NormalGC, 0, y_offset,
 		    mr->width, Scr->EntryHeight);
 
-		FBF(mi->fore, mi->back, Scr->MenuFont.font->fid);
+		MyFont_ChangeGC(mi->fore, mi->back, &Scr->MenuFont);
 		gc = Scr->NormalGC;
 	    }
 	    else
 		gc = Scr->MenuGC;
 
-	    XDrawString(dpy, mr->w, gc, mi->x,
-		text_y, mi->item, mi->strlen);
+	    MyFont_DrawString(dpy, mr->w, &Scr->MenuFont, gc, 
+		    mi->x, text_y, mi->item, mi->strlen);
+
 	}
 
 	if (mi->func == F_MENU)
@@ -461,15 +464,15 @@ int exposure;
 	    XDrawLine(dpy, mr->w, Scr->NormalGC, 0, y, mr->width, y);
 	}
 
-	FBF(mi->fore, mi->back, Scr->MenuFont.font->fid);
+	MyFont_ChangeGC(mi->fore, mi->back, &Scr->MenuFont);
 	/* finally render the title */
-	XDrawString(dpy, mr->w, Scr->NormalGC, mi->x,
+	MyFont_DrawString(dpy, mr->w, &Scr->MenuFont, Scr->NormalGC, mi->x,
 	    text_y, mi->item, mi->strlen);
     }
 }
     
 
-
+void
 PaintMenu(mr, e)
 MenuRoot *mr;
 XEvent *e;
@@ -496,6 +499,7 @@ XEvent *e;
 
 static Bool fromMenu;
 
+void
 UpdateMenu()
 {
     MenuItem *mi;
@@ -753,7 +757,7 @@ AddToMenu(menu, item, action, sub, func, fore, back)
     tmp->func = func;
 
     if (!Scr->HaveFonts) CreateFonts();
-    width = XTextWidth(Scr->MenuFont.font, item, tmp->strlen);
+    width = MyFont_TextWidth(&Scr->MenuFont, item, tmp->strlen);
     if (width <= 0)
 	width = 1;
     if (width > menu->width)
@@ -782,7 +786,7 @@ AddToMenu(menu, item, action, sub, func, fore, back)
 }
 
 
-
+void
 MakeMenus()
 {
     MenuRoot *mr;
@@ -797,7 +801,7 @@ MakeMenus()
 }
 
 
-
+void
 MakeMenu(mr)
 MenuRoot *mr;
 {
@@ -831,7 +835,7 @@ MenuRoot *mr;
 		cur->x = 5;
 	    else
 	    {
-		cur->x = width - XTextWidth(Scr->MenuFont.font, cur->item,
+		cur->x = width - MyFont_TextWidth(&Scr->MenuFont, cur->item,
 		    cur->strlen);
 		cur->x /= 2;
 	    }
@@ -1002,7 +1006,8 @@ MenuRoot *mr;
  ***********************************************************************
  */
 
-Bool PopUpMenu (menu, x, y, center)
+Bool 
+PopUpMenu (menu, x, y, center)
     MenuRoot *menu;
     int x, y;
     Bool center;
@@ -1011,7 +1016,7 @@ Bool PopUpMenu (menu, x, y, center)
     TwmWindow **WindowNames;
     TwmWindow *tmp_win2,*tmp_win3;
     int i;
-    int (*compar)() = 
+    int (*compar)(const char *, const char *) = 
       (Scr->CaseSensitive ? strcmp : XmuCompareISOLatin1);
 
     if (!menu) return False;
@@ -1135,7 +1140,7 @@ Bool PopUpMenu (menu, x, y, center)
  *
  ***********************************************************************
  */
-
+void
 PopDownMenu()
 {
     MenuRoot *tmp;
@@ -1199,7 +1204,8 @@ FindMenuRoot(name)
 
 
 
-static Bool belongs_to_twm_window (t, w)
+static Bool 
+belongs_to_twm_window (t, w)
     register TwmWindow *t;
     register Window w;
 {
@@ -1235,22 +1241,21 @@ extern int AddingY;
 extern int AddingW;
 extern int AddingH;
 
-void resizeFromCenter(w, tmp_win)
+void 
+resizeFromCenter(w, tmp_win)
      Window w;
      TwmWindow *tmp_win;
 {
   int lastx, lasty, width, height, bw2;
   int namelen;
-  int stat;
   XEvent event;
-  Window junk;
 
   namelen = strlen (tmp_win->name);
   bw2 = tmp_win->frame_bw * 2;
   AddingW = tmp_win->attr.width + bw2;
   AddingH = tmp_win->attr.height + tmp_win->title_height + bw2;
-  width = (SIZE_HINDENT + XTextWidth (Scr->SizeFont.font,
-				      tmp_win->name, namelen));
+  width = (SIZE_HINDENT + MyFont_TextWidth (&Scr->SizeFont,
+					     tmp_win->name, namelen));
   height = Scr->SizeFont.height + SIZE_VINDENT * 2;
   XGetGeometry(dpy, w, &JunkRoot, &origDragX, &origDragY,
 	       (unsigned int *)&DragWidth, (unsigned int *)&DragHeight, 
@@ -1262,11 +1267,11 @@ void resizeFromCenter(w, tmp_win)
 		 &AddingX, &AddingY, &JunkMask);
 /*****
   Scr->SizeStringOffset = width +
-    XTextWidth(Scr->SizeFont.font, ": ", 2);
+    MyFont_TextWidth(&Scr->SizeFont, ": ", 2);
   XResizeWindow (dpy, Scr->SizeWindow, Scr->SizeStringOffset +
 		 Scr->SizeStringWidth, height);
-  XDrawImageString (dpy, Scr->SizeWindow, Scr->NormalGC, width,
-		    SIZE_VINDENT + Scr->SizeFont.font->ascent,
+  MyFont_DrawImageString (dpy, Scr->SizeWindow, &Scr->SizeFont, Scr->NormalGC,
+		    width, SIZE_VINDENT + Scr->SizeFont.ascent,
 		    ": ", 2);
 *****/
   lastx = -10000;
@@ -1384,7 +1389,6 @@ ExecuteFunction(func, action, w, tmp_win, eventp, context, pulldown)
     int do_next_action = TRUE;
     int moving_icon = FALSE;
     Bool fromtitlebar = False;
-    extern int ConstrainedMoveTime;
 
     RootFunction = 0;
     if (Cancel)
@@ -1408,6 +1412,8 @@ ExecuteFunction(func, action, w, tmp_win, eventp, context, pulldown)
     case F_WARPTO:
     case F_WARPRING:
     case F_WARPTOICONMGR:
+    case F_WARPNEXT:
+    case F_WARPPREV:
     case F_COLORMAP:
 	break;
     default:
@@ -1430,8 +1436,6 @@ ExecuteFunction(func, action, w, tmp_win, eventp, context, pulldown)
 
     case F_RESTART:
     {
-	extern SmcConn smcConn;
-
 	XSync (dpy, 0);
 	Reborder (eventp->xbutton.time);
 	XSync (dpy, 0);
@@ -2020,7 +2024,8 @@ ExecuteFunction(func, action, w, tmp_win, eventp, context, pulldown)
 		InstallWindowColormaps (0, tmp_win);
 		if (tmp_win->hilite_w) XMapWindow (dpy, tmp_win->hilite_w);
 		SetBorder (tmp_win, True);
-		SetFocus (tmp_win, eventp->xbutton.time);
+		if (!tmp_win->wmhints || tmp_win->wmhints->input)
+		    SetFocus (tmp_win, eventp->xbutton.time);
 		Scr->FocusRoot = FALSE;
 		Scr->Focus = tmp_win;
 	    }
@@ -2139,6 +2144,40 @@ ExecuteFunction(func, action, w, tmp_win, eventp, context, pulldown)
 	}
 	break;
 
+    case F_WARPPREV:
+    case F_WARPNEXT:
+	{
+		register TwmWindow *t;
+		static TwmWindow *savedwarp = NULL;
+		TwmWindow *of, *l, *n;
+		int c=0;
+
+#define wseq(w) (func == F_WARPNEXT ? (w)->next : (w)->prev)
+#define nwin(w) ((w) && (n=wseq(w)) != NULL && n != &Scr->TwmRoot ? n : l)
+#define bwin(w) (!(w)||(w)->iconmgr||(w)==of||!(Scr->WarpUnmapped||(w)->mapped))
+
+		of=(Scr->Focus ? Scr->Focus : &Scr->TwmRoot);
+
+		for(t=Scr->TwmRoot.next; t; t=t->next) if(!bwin(t)) break;
+		if(!t) break;	/* no windows we can use */
+
+		if(func == F_WARPPREV) for(l=of; l->next; l=l->next) ;
+		else l = Scr->TwmRoot.next;
+
+		for(t=of; bwin(t) && c < 2; t=nwin(t)) if(t == of) c++;
+
+		if(bwin(t) || c >= 2) Bell(XkbBI_MinorError,0,None);
+		else {
+			if(of && of == savedwarp) {
+				Iconify(of, 0, 0);
+				savedwarp = NULL;
+			}
+			if(!t->mapped) savedwarp = t; else savedwarp = NULL;
+			WarpThere(t);
+		}
+		break;
+	}
+
     case F_WARPTO:
 	{
 	    register TwmWindow *t;
@@ -2275,7 +2314,7 @@ ExecuteFunction(func, action, w, tmp_win, eventp, context, pulldown)
 	break;
 
     case F_QUIT:
-	Done();
+	Done(0);
 	break;
 
     case F_PRIORITY:
@@ -2338,7 +2377,7 @@ Cursor cursor;
  *
  ***********************************************************************
  */
-
+void
 ReGrab()
 {
     XGrabPointer(dpy, Scr->Root, True,
@@ -2360,7 +2399,7 @@ ReGrab()
  *
  ***********************************************************************
  */
-
+Bool
 NeedToDefer(root)
 MenuRoot *root;
 {
@@ -2410,7 +2449,8 @@ MenuRoot *root;
  */
 
 #if defined(sun) && defined(SVR4)
-static int System (s)
+static int 
+System (s)
     char *s;
 {
     int pid, status;
@@ -2490,6 +2530,7 @@ FocusOnRoot()
     Scr->FocusRoot = TRUE;
 }
 
+void
 DeIconify(tmp_win)
 TwmWindow *tmp_win;
 {
@@ -2567,7 +2608,7 @@ TwmWindow *tmp_win;
 }
 
 
-
+void
 Iconify(tmp_win, def_x, def_y)
 TwmWindow *tmp_win;
 int def_x, def_y;
@@ -2661,8 +2702,9 @@ int def_x, def_y;
 
 
 
-static void Identify (t)
-TwmWindow *t;
+static void 
+Identify (t)
+    TwmWindow *t;
 {
     int i, n, twidth, width, height;
     int x, y;
@@ -2704,7 +2746,7 @@ TwmWindow *t;
     width = 1;
     for (i = 0; i < n; i++)
     {
-	twidth = XTextWidth(Scr->DefaultFont.font, Info[i],
+	twidth = MyFont_TextWidth(&Scr->DefaultFont, Info[i], 
 	    strlen(Info[i]));
 	if (twidth > width)
 	    width = twidth;
@@ -2731,10 +2773,10 @@ TwmWindow *t;
 }
 
 
-
+void
 SetMapStateProp(tmp_win, state)
-TwmWindow *tmp_win;
-int state;
+    TwmWindow *tmp_win;
+    int state;
 {
     unsigned long data[2];		/* "suggested" by ICCCM version 1 */
   
@@ -2748,7 +2790,8 @@ int state;
 
 
 
-Bool GetWMState (w, statep, iwp)
+Bool 
+GetWMState (w, statep, iwp)
     Window w;
     int *statep;
     Window *iwp;
@@ -2775,7 +2818,7 @@ Bool GetWMState (w, statep, iwp)
 }
 
 
-
+void
 WarpToScreen (n, inc)
     int n, inc;
 {
@@ -2820,7 +2863,7 @@ WarpToScreen (n, inc)
 /*
  * BumpWindowColormap - rotate our internal copy of WM_COLORMAP_WINDOWS
  */
-
+void
 BumpWindowColormap (tmp, inc)
     TwmWindow *tmp;
     int inc;
@@ -2834,9 +2877,9 @@ BumpWindowColormap (tmp, inc)
 	cwins = (ColormapWindow **) malloc(sizeof(ColormapWindow *)*
 					   tmp->cmaps.number_cwins);
 	if (cwins) {		
-	    if (previously_installed =
+	    if ((previously_installed =
 		/* SUPPRESS 560 */(Scr->cmapInfo.cmaps == &tmp->cmaps &&
-	        tmp->cmaps.number_cwins)) {
+	        tmp->cmaps.number_cwins))) {
 		for (i = tmp->cmaps.number_cwins; i-- > 0; )
 		    tmp->cmaps.cwins[i]->colormap->state = 0;
 	    }
@@ -2866,7 +2909,7 @@ BumpWindowColormap (tmp, inc)
 }
 
 
-
+void
 HideIconManager ()
 {
     SetMapStateProp (Scr->iconmgr.twm_win, WithdrawnState);
@@ -2879,7 +2922,7 @@ HideIconManager ()
 
 
 
-
+void
 SetBorder (tmp, onoroff)
     TwmWindow *tmp;
     Bool onoroff;
@@ -2898,7 +2941,7 @@ SetBorder (tmp, onoroff)
 }
 
 
-
+void
 DestroyMenu (menu)
     MenuRoot *menu;
 {
@@ -2923,7 +2966,8 @@ DestroyMenu (menu)
 /*
  * warping routines
  */
-void WarpAlongRing (ev, forward)
+void 
+WarpAlongRing (ev, forward)
     XButtonEvent *ev;
     Bool forward;
 {
@@ -2970,7 +3014,8 @@ void WarpAlongRing (ev, forward)
 
 
 
-void WarpToWindow (t)
+void 
+WarpToWindow (t)
     TwmWindow *t;
 {
     int x, y;
@@ -3000,7 +3045,8 @@ void WarpToWindow (t)
  *     data[0]		message atom
  *     data[1]		time stamp
  */
-static void send_clientmessage (w, a, timestamp)
+static void 
+send_clientmessage (w, a, timestamp)
     Window w;
     Atom a;
     Time timestamp;
@@ -3016,6 +3062,7 @@ static void send_clientmessage (w, a, timestamp)
     XSendEvent (dpy, w, False, 0L, (XEvent *) &ev);
 }
 
+void
 SendDeleteWindowMessage (tmp, timestamp)
     TwmWindow *tmp;
     Time timestamp;
@@ -3023,6 +3070,7 @@ SendDeleteWindowMessage (tmp, timestamp)
     send_clientmessage (tmp->w, _XA_WM_DELETE_WINDOW, timestamp);
 }
 
+void
 SendSaveYourselfMessage (tmp, timestamp)
     TwmWindow *tmp;
     Time timestamp;
@@ -3030,7 +3078,7 @@ SendSaveYourselfMessage (tmp, timestamp)
     send_clientmessage (tmp->w, _XA_WM_SAVE_YOURSELF, timestamp);
 }
 
-
+void
 SendTakeFocusMessage (tmp, timestamp)
     TwmWindow *tmp;
     Time timestamp;
