@@ -1,16 +1,9 @@
-/* $XConsortium: Login.c,v 1.42 94/04/17 20:03:53 gildea Exp $ */
-/* $XFree86: xc/programs/xdm/greeter/Login.c,v 3.1.4.2 1999/07/23 13:23:19 hohndel Exp $ */
+/* $Xorg: Login.c,v 1.3 2000/08/17 19:54:17 cpqbld Exp $ */
 /*
 
-Copyright (c) 1988  X Consortium
+Copyright 1988, 1998  The Open Group
 
-Permission is hereby granted, free of charge, to any person obtaining
-a copy of this software and associated documentation files (the
-"Software"), to deal in the Software without restriction, including
-without limitation the rights to use, copy, modify, merge, publish,
-distribute, sublicense, and/or sell copies of the Software, and to
-permit persons to whom the Software is furnished to do so, subject to
-the following conditions:
+All Rights Reserved.
 
 The above copyright notice and this permission notice shall be included
 in all copies or substantial portions of the Software.
@@ -18,17 +11,18 @@ in all copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
-IN NO EVENT SHALL THE X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR
+IN NO EVENT SHALL THE OPEN GROUP BE LIABLE FOR ANY CLAIM, DAMAGES OR
 OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 
-Except as contained in this notice, the name of the X Consortium shall
+Except as contained in this notice, the name of The Open Group shall
 not be used in advertising or otherwise to promote the sale, use or
 other dealings in this Software without prior written authorization
-from the X Consortium.
+from The Open Group.
 
 */
+/* $XFree86: xc/programs/xdm/greeter/Login.c,v 3.12 2001/01/17 23:45:25 dawes Exp $ */
 
 /*
  * xdm - display manager daemon
@@ -44,14 +38,36 @@ from the X Consortium.
 # include <X11/Xfuncs.h>
 
 # include <stdio.h>
+#ifdef XPM
+# include <time.h>
+#endif /* XPM */
 
 # include "dm.h"
+# include "dm_error.h"
 # include "greet.h"
 # include "LoginP.h"
 
+#ifdef XPM
+#include <sys/stat.h>
+#include <unistd.h>
+#include <X11/Xlib.h>
+#include <X11/xpm.h>
+#include <X11/extensions/shape.h>
+#include <X11/cursorfont.h>
+#endif /* XPM */
+
+#ifdef USE_XINERAMA
+#include <X11/extensions/Xinerama.h>
+#endif
+
+static void RedrawFail (LoginWidget w);
+static void ResetLogin (LoginWidget w);
+static void failTimeout (XtPointer client_data, XtIntervalId * id);
+
 #define offset(field) XtOffsetOf(LoginRec, login.field)
 #define goffset(field) XtOffsetOf(WidgetRec, core.field)
 
+
 static XtResource resources[] = {
     {XtNwidth, XtCWidth, XtRDimension, sizeof(Dimension),
 	goffset(width), XtRImmediate,	(XtPointer) 0},
@@ -69,6 +85,37 @@ static XtResource resources[] = {
         offset(greetpixel), XtRString,	XtDefaultForeground},
     {XtNfailColor, XtCForeground, XtRPixel, sizeof (Pixel),
 	offset(failpixel), XtRString,	XtDefaultForeground},
+
+#ifdef XPM
+/* added by Caolan McNamara */
+	{XtNlastEventTime, XtCLastEventTime, XtRInt , sizeof (int),
+	offset(lastEventTime), XtRImmediate,	(XtPointer)0},
+/* end (caolan) */
+
+/* added by Ivan Griffin (ivan.griffin@ul.ie) */
+        {XtNlogoFileName, XtCLogoFileName, XtRString, sizeof(char*),
+        offset(logoFileName), XtRImmediate, (XtPointer)0},
+        {XtNuseShape, XtCUseShape, XtRBoolean, sizeof(Boolean),
+        offset(useShape), XtRImmediate, (XtPointer) True},
+        {XtNlogoPadding, XtCLogoPadding, XtRInt, sizeof(int),
+        offset(logoPadding), XtRImmediate, (XtPointer) 5},
+/* end (ivan) */
+
+
+/* added by Amit Margalit */
+    {XtNhiColor, XtCForeground, XtRPixel, sizeof (Pixel),
+	offset(hipixel), XtRString,	XtDefaultForeground},
+    {XtNshdColor, XtCForeground, XtRPixel, sizeof (Pixel),
+	offset(shdpixel), XtRString,	XtDefaultForeground},
+    {XtNframeWidth, XtCFrameWidth, XtRInt, sizeof(int),
+        offset(outframewidth), XtRImmediate, (XtPointer) 1},
+    {XtNinnerFramesWidth, XtCFrameWidth, XtRInt, sizeof(int),
+        offset(inframeswidth), XtRImmediate, (XtPointer) 1},
+    {XtNsepWidth, XtCFrameWidth, XtRInt, sizeof(int),
+        offset(sepwidth), XtRImmediate, (XtPointer) 1},
+/* end (amit) */
+#endif /* XPM */
+
     {XtNfont, XtCFont, XtRFontStruct, sizeof (XFontStruct *),
     	offset (font), XtRString,	"*-new century schoolbook-medium-r-normal-*-180-*"},
     {XtNpromptFont, XtCFont, XtRFontStruct, sizeof (XFontStruct *),
@@ -98,7 +145,9 @@ static XtResource resources[] = {
     {XtNallowAccess, XtCAllowAccess, XtRBoolean, sizeof (Boolean),
 	offset(allow_access), XtRImmediate, False },
     {XtNallowNullPasswd, XtCAllowNullPasswd, XtRBoolean, sizeof (Boolean),
-	offset(allow_null_passwd), XtRImmediate, False}
+	offset(allow_null_passwd), XtRImmediate, False},
+    {XtNallowRootLogin, XtCAllowRootLogin, XtRBoolean, sizeof(Boolean),
+     offset(allow_root_login), XtRImmediate, (XtPointer) True}
 };
 
 #undef offset
@@ -119,22 +168,43 @@ static XtResource resources[] = {
 
 # define Y_INC(w)	max (TEXT_Y_INC(w), PROMPT_Y_INC(w))
 
+#ifndef XPM
 # define LOGIN_PROMPT_W(w) (XTextWidth (w->login.promptFont,\
 				 w->login.namePrompt,\
 				 strlen (w->login.namePrompt)))
+#else
+# define LOGIN_PROMPT_W(w) (XTextWidth (w->login.promptFont,\
+				 w->login.namePrompt,\
+				 strlen (w->login.namePrompt)) + \
+				 w->login.inframeswidth)
+#endif /* XPM */
+#ifndef XPM
 # define PASS_PROMPT_W(w) (XTextWidth (w->login.promptFont,\
 				 w->login.passwdPrompt,\
 				 strlen (w->login.passwdPrompt)))
+#else
+# define PASS_PROMPT_W(w) (XTextWidth (w->login.promptFont,\
+				 w->login.passwdPrompt,\
+				 strlen (w->login.passwdPrompt)) + \
+				 w->login.inframeswidth)
+#endif /* XPM */
 # define PROMPT_W(w)	(max(LOGIN_PROMPT_W(w), PASS_PROMPT_W(w)))
 # define GREETING(w)	((w)->login.secure_session  && !(w)->login.allow_access ?\
 				(w)->login.greeting : (w)->login.unsecure_greet)
 # define GREET_X(w)	((int)(w->core.width - XTextWidth (w->login.greetFont,\
 			  GREETING(w), strlen (GREETING(w)))) / 2)
 # define GREET_Y(w)	(GREETING(w)[0] ? 2 * GREET_Y_INC (w) : 0)
+#ifndef XPM
 # define GREET_W(w)	(max (XTextWidth (w->login.greetFont,\
 			      w->login.greeting, strlen (w->login.greeting)), \
 			      XTextWidth (w->login.greetFont,\
 			      w->login.unsecure_greet, strlen (w->login.unsecure_greet))))
+#else
+# define GREET_W(w)	(max (XTextWidth (w->login.greetFont,\
+			      w->login.greeting, strlen (w->login.greeting)), \
+			      XTextWidth (w->login.greetFont,\
+			      w->login.unsecure_greet, strlen (w->login.unsecure_greet)))) + w->login.logoWidth + (2*w->login.logoPadding)
+#endif /* XPM */
 # define LOGIN_X(w)	(2 * PROMPT_X_INC(w))
 # define LOGIN_Y(w)	(GREET_Y(w) + GREET_Y_INC(w) +\
 			 w->login.greetFont->max_bounds.ascent + Y_INC(w))
@@ -142,7 +212,11 @@ static XtResource resources[] = {
 # define LOGIN_H(w)	(3 * Y_INC(w) / 2)
 # define LOGIN_TEXT_X(w)(LOGIN_X(w) + PROMPT_W(w))
 # define PASS_X(w)	(LOGIN_X(w))
+#ifndef XPM
 # define PASS_Y(w)	(LOGIN_Y(w) + 8 * Y_INC(w) / 5)
+#else
+# define PASS_Y(w)	(LOGIN_Y(w) + 10 * Y_INC(w) / 5)
+#endif /* XPM */
 # define PASS_W(w)	(LOGIN_W(w))
 # define PASS_H(w)	(LOGIN_H(w))
 # define PASS_TEXT_X(w)	(PASS_X(w) + PROMPT_W(w))
@@ -150,26 +224,25 @@ static XtResource resources[] = {
 				w->login.fail, strlen (w->login.fail))) / 2)
 # define FAIL_Y(w)	(PASS_Y(w) + 2 * FAIL_Y_INC (w) +\
 			w->login.failFont->max_bounds.ascent)
+#ifndef XPM
 # define FAIL_W(w)	(XTextWidth (w->login.failFont,\
 			 w->login.fail, strlen (w->login.fail)))
+#else
+# define FAIL_W(w)	(XTextWidth (w->login.failFont,\
+			 w->login.fail, strlen (w->login.fail))) + w->login.logoWidth + (2*w->login.logoPadding)
+#endif /* XPM */
 
 # define PAD_X(w)	(2 * (LOGIN_X(w) + max (GREET_X_INC(w), FAIL_X_INC(w))))
 
 # define PAD_Y(w)	(max (max (Y_INC(w), GREET_Y_INC(w)),\
 			     FAIL_Y_INC(w)))
 	
-static void Initialize(), Realize(), Destroy(), Redisplay();
-static Boolean SetValues();
-static void draw_it ();
-
-#ifndef __QNX__
-static int max (a,b) { return a > b ? a : b; }
-#endif /* QNX has max() defines already */
+#ifndef max
+static int max (int a, int b) { return a > b ? a : b; }
+#endif
 
 static void
-EraseName (w, cursor)
-    LoginWidget	w;
-    int		cursor;
+EraseName (LoginWidget w, int cursor)
 {
     int	x;
 
@@ -181,9 +254,7 @@ EraseName (w, cursor)
 }
 
 static void
-DrawName (w, cursor)
-    LoginWidget	w;
-    int		cursor;
+DrawName (LoginWidget w, int cursor)
 {
     int	x;
 
@@ -192,12 +263,16 @@ DrawName (w, cursor)
 	x += XTextWidth (w->login.font, w->login.data.name, cursor);
     XDrawString (XtDisplay(w), XtWindow (w), w->login.textGC, x, LOGIN_Y(w),
 		w->login.data.name + cursor, strlen (w->login.data.name + cursor));
+
+#ifdef XPM
+	/*as good a place as any Caolan begin*/
+	w->login.lastEventTime = time(NULL);
+	/*as good a place as any Caolan end*/
+#endif /* XPM */
 }
 
 static void
-realizeCursor (w, gc)
-    LoginWidget	w;
-    GC		gc;
+realizeCursor (LoginWidget w, GC gc)
 {
     int	x, y;
     int height, width;
@@ -221,12 +296,31 @@ realizeCursor (w, gc)
 	return;
     }
     XFillRectangle (XtDisplay (w), XtWindow (w), gc,
+#ifndef XPM
 		    x, y - w->login.font->max_bounds.ascent, width, height);
+#else
+		    x, y+1 - w->login.font->max_bounds.ascent, width, height-1);
+    XDrawPoint     (XtDisplay (w), XtWindow (w), gc,
+    		    x-1 , y - w->login.font->max_bounds.ascent);
+    XDrawPoint     (XtDisplay (w), XtWindow (w), gc,
+    		    x+1 , y - w->login.font->max_bounds.ascent);
+    XDrawPoint     (XtDisplay (w), XtWindow (w), gc,
+    		    x-1 , y - w->login.font->max_bounds.ascent+height);
+    XDrawPoint     (XtDisplay (w), XtWindow (w), gc,
+    		    x+1 , y - w->login.font->max_bounds.ascent+height);
+    XDrawPoint     (XtDisplay (w), XtWindow (w), gc,
+    		    x-2 , y - w->login.font->max_bounds.ascent);
+    XDrawPoint     (XtDisplay (w), XtWindow (w), gc,
+    		    x+2 , y - w->login.font->max_bounds.ascent);
+    XDrawPoint     (XtDisplay (w), XtWindow (w), gc,
+    		    x-2 , y - w->login.font->max_bounds.ascent+height);
+    XDrawPoint     (XtDisplay (w), XtWindow (w), gc,
+    		    x+2 , y - w->login.font->max_bounds.ascent+height);
+#endif /* XPM */
 }
 
 static void
-EraseFail (w)
-    LoginWidget	w;
+EraseFail (LoginWidget w)
 {
     int x = FAIL_X(w);
     int y = FAIL_Y(w);
@@ -242,31 +336,26 @@ EraseFail (w)
 }
 
 static void
-XorCursor (w)
-    LoginWidget	w;
+XorCursor (LoginWidget w)
 {
     realizeCursor (w, w->login.xorGC);
 }
 
 static void
-RemoveFail (w)
-    LoginWidget	w;
+RemoveFail (LoginWidget w)
 {
     if (w->login.failUp)
 	EraseFail (w);
 }
 
 static void
-EraseCursor (w)
-    LoginWidget (w);
+EraseCursor (LoginWidget w)
 {
     realizeCursor (w, w->login.bgGC);
 }
 
 /*ARGSUSED*/
-void failTimeout (client_data, id)
-    XtPointer	client_data;
-    XtIntervalId *	id;
+static void failTimeout (XtPointer client_data, XtIntervalId * id)
 {
     LoginWidget	w = (LoginWidget)client_data;
 
@@ -274,8 +363,8 @@ void failTimeout (client_data, id)
     EraseFail (w);
 }
 
-DrawFail (ctx)
-    Widget	ctx;
+void
+DrawFail (Widget ctx)
 {
     LoginWidget	w;
 
@@ -293,8 +382,8 @@ DrawFail (ctx)
     }
 }
 
-RedrawFail (w)
-    LoginWidget w;
+static void
+RedrawFail (LoginWidget w)
 {
     int x = FAIL_X(w);
     int y = FAIL_Y(w);
@@ -306,13 +395,101 @@ RedrawFail (w)
 }
 
 static void
-draw_it (w)
-    LoginWidget	w;
+draw_it (LoginWidget w)
 {
+#ifdef XPM
+    int i,in_frame_x,in_login_y,in_pass_y,in_width,in_height;
+    int gr_line_x, gr_line_y, gr_line_w;
+#endif /* XPM */
+
     EraseCursor (w);
+
+#ifdef XPM
+    if( (w->login.outframewidth) < 1 )
+      w->login.outframewidth = 1;
+    for(i=1;i<=(w->login.outframewidth);i++)
+    {
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.hiGC,
+    		i-1,i-1,w->core.width-i,i-1);
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.hiGC,
+    		i-1,i-1,i-1,w->core.height-i);
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.shdGC,
+    		w->core.width-i,i-1,w->core.width-i,w->core.height-i);
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.shdGC,
+    		i-1,w->core.height-i,w->core.width-i,w->core.height-i);
+    }
+    
+    /* make separator line */
+    gr_line_x = w->login.outframewidth + w->login.logoPadding;
+    gr_line_y = GREET_Y(w) + GREET_Y_INC(w);
+    gr_line_w = w->core.width - 2*(w->login.outframewidth) -
+        (w->login.logoWidth + 3*(w->login.logoPadding));
+ 
+    for(i=1;i<=(w->login.sepwidth);i++)
+    {
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.shdGC,
+        gr_line_x,           gr_line_y + i-1,
+        gr_line_x+gr_line_w, gr_line_y + i-1);
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.hiGC,
+        gr_line_x,           gr_line_y + 2*(w->login.inframeswidth) -i,
+        gr_line_x+gr_line_w, gr_line_y + 2*(w->login.inframeswidth) -i);
+    }
+    
+    in_frame_x = LOGIN_TEXT_X(w) - w->login.inframeswidth - 3;
+    in_login_y = LOGIN_Y(w) - w->login.inframeswidth - 1 - TEXT_Y_INC(w);
+    in_pass_y  = PASS_Y(w) - w->login.inframeswidth - 1 - TEXT_Y_INC(w);
+ 
+    in_width = LOGIN_W(w) - PROMPT_W(w) -
+        (w->login.logoWidth + 2*(w->login.logoPadding));
+    in_height = LOGIN_H(w) + w->login.inframeswidth + 2;
+
+    for(i=1;i<=(w->login.inframeswidth);i++)
+    {
+      /* Make top/left sides */
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.shdGC,
+	in_frame_x + i-1,             in_login_y + i-1,
+	in_frame_x + in_width-i,      in_login_y + i-1); 
+
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.shdGC,
+	in_frame_x + i-1,             in_login_y + i-1,
+	in_frame_x + i-1,             in_login_y + in_height-i); 
+
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.hiGC,
+        in_frame_x + in_width-i,      in_login_y + i-1,
+        in_frame_x + in_width-i,      in_login_y + in_height-i); 
+                
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.hiGC,
+	in_frame_x + i-1,             in_login_y + in_height-i,
+	in_frame_x + in_width-i,      in_login_y + in_height-i);
+
+      /* Make bottom/right sides */
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.shdGC,
+	in_frame_x + i-1,             in_pass_y + i-1,
+	in_frame_x + in_width-i,      in_pass_y + i-1); 
+
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.shdGC,
+	in_frame_x + i-1,             in_pass_y + i-1,
+	in_frame_x + i-1,             in_pass_y + in_height-i); 
+
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.hiGC,
+        in_frame_x + in_width-i,      in_pass_y + i-1,
+        in_frame_x + in_width-i,      in_pass_y + in_height-i); 
+                
+      XDrawLine(XtDisplay (w), XtWindow (w), w->login.hiGC,
+	in_frame_x + i-1,             in_pass_y + in_height-i,
+	in_frame_x + in_width-i,      in_pass_y + in_height-i);
+    }
+#endif /* XPM */
+
     if (GREETING(w)[0])
 	    XDrawString (XtDisplay (w), XtWindow (w), w->login.greetGC,
+#ifndef XPM
 			GREET_X(w), GREET_Y(w),
+#else
+			GREET_X(w) -
+                            ((w->login.logoWidth/2) + w->login.logoPadding),
+                        GREET_Y(w),
+#endif /* XPM */
 			GREETING(w), strlen (GREETING(w)));
     XDrawString (XtDisplay (w), XtWindow (w), w->login.promptGC,
 		LOGIN_X(w), LOGIN_Y(w),
@@ -341,11 +518,7 @@ draw_it (w)
 
 /*ARGSUSED*/
 static void
-DeleteBackwardChar (ctxw, event, params, num_params)
-    Widget ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+DeleteBackwardChar (Widget ctxw, XEvent *event, String *params, Cardinal *num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
@@ -371,11 +544,7 @@ DeleteBackwardChar (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-DeleteForwardChar (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+DeleteForwardChar (Widget ctxw, XEvent *event, String *params, Cardinal *num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
@@ -402,11 +571,11 @@ DeleteForwardChar (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-MoveBackwardChar (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+MoveBackwardChar (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget	ctx = (LoginWidget)ctxw;
 
@@ -419,11 +588,11 @@ MoveBackwardChar (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-MoveForwardChar (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+MoveForwardChar (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
@@ -444,11 +613,11 @@ MoveForwardChar (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-MoveToBegining (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+MoveToBegining (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
@@ -460,11 +629,11 @@ MoveToBegining (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-MoveToEnd (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+MoveToEnd (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
@@ -483,11 +652,11 @@ MoveToEnd (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-EraseToEndOfLine (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+EraseToEndOfLine (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
@@ -507,11 +676,11 @@ EraseToEndOfLine (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-EraseLine (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+EraseLine (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     MoveToBegining (ctxw, event, params, num_params);
     EraseToEndOfLine (ctxw, event, params, num_params);
@@ -519,11 +688,11 @@ EraseLine (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-FinishField (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+FinishField (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
@@ -543,13 +712,36 @@ FinishField (ctxw, event, params, num_params)
     XorCursor (ctx);
 }
 
+#ifdef XPM
 /*ARGSUSED*/
 static void
-AllowAccess (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+TabField(Widget ctxw, XEvent *event, String *params, Cardinal *num_params)
+{
+    LoginWidget ctx = (LoginWidget)ctxw;
+
+    XorCursor (ctx);
+    RemoveFail (ctx);
+    switch (ctx->login.state) {
+    case GET_NAME:
+	ctx->login.state = GET_PASSWD;
+	ctx->login.cursor = 0;
+	break;
+    case GET_PASSWD:
+	ctx->login.state = GET_NAME;
+	ctx->login.cursor = 0;
+	break;
+    }
+    XorCursor (ctx);
+}
+#endif /* XPM */
+
+/*ARGSUSED*/
+static void
+AllowAccess (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
     Arg	arglist[1];
@@ -564,11 +756,11 @@ AllowAccess (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-SetSessionArgument (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+SetSessionArgument (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
@@ -587,11 +779,11 @@ SetSessionArgument (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-RestartSession (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+RestartSession (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
@@ -605,11 +797,11 @@ RestartSession (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-AbortSession (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+AbortSession (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
@@ -623,11 +815,11 @@ AbortSession (ctxw, event, params, num_params)
 
 /*ARGSUSED*/
 static void
-AbortDisplay (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+AbortDisplay (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
@@ -639,8 +831,8 @@ AbortDisplay (ctxw, event, params, num_params)
     XorCursor (ctx);
 }
 
-ResetLogin (w)
-    LoginWidget	w;
+static void
+ResetLogin (LoginWidget w)
 {
     EraseName (w, 0);
     w->login.cursor = 0;
@@ -667,29 +859,33 @@ InitI18N(Widget ctxw)
     }
 
     ctx->login.xic = XCreateIC(xim,
-	 XNInputStyle, (XIMPreeditNothing|XIMStatusNothing),
-	 XNClientWindow, ctx->core.window,
-	 XNFocusWindow,  ctx->core.window, NULL);
+	XNInputStyle, (XIMPreeditNothing|XIMStatusNothing),
+	XNClientWindow, ctx->core.window,
+	XNFocusWindow,  ctx->core.window, NULL);
 
     if (!ctx->login.xic) {
-        LogError("Failed to create input context\n");
-        XCloseIM(xim);
+	LogError("Failed to create input context\n");
+	XCloseIM(xim);
     }
     return;
 }
 
 /* ARGSUSED */
 static void
-InsertChar (ctxw, event, params, num_params)
-    Widget	ctxw;
-    XEvent	*event;
-    String	*params;
-    Cardinal	*num_params;
+InsertChar (
+    Widget	ctxw,
+    XEvent	*event,
+    String	*params,
+    Cardinal	*num_params)
 {
     LoginWidget ctx = (LoginWidget)ctxw;
 
     char strbuf[128];
+#ifndef XPM
     int  len;
+#else
+    int  len,pixels;
+#endif /* XPM */
     KeySym  keysym = 0;
 
     if (ctx->login.xic) {
@@ -703,6 +899,14 @@ InsertChar (ctxw, event, params, num_params)
     }
     strbuf[len] = '\0';
 
+#ifdef XPM
+    pixels = 3 + ctx->login.font->max_bounds.width * len +
+    	     XTextWidth(ctx->login.font,
+    	     		ctx->login.data.name,
+    	     		strlen(ctx->login.data.name));
+    	     	/* pixels to be added */
+#endif /* XPM */
+
     /*
      * Note: You can override this default key handling
      * by the settings in the translation table
@@ -754,13 +958,24 @@ InsertChar (ctxw, event, params, num_params)
 
     switch (ctx->login.state) {
     case GET_NAME:
+#ifndef XPM
 	if (len + (int)strlen(ctx->login.data.name) >= NAME_LEN - 1)
+#else
+	if (
+	        (len + (int)strlen(ctx->login.data.name) >= NAME_LEN - 1)/* &&
+		(pixels <= LOGIN_W(ctx) - PROMPT_W(ctx))*/
+	   )
+#endif /* XPM */
 	    len = NAME_LEN - strlen(ctx->login.data.name) - 2;
     case GET_PASSWD:
 	if (len + (int)strlen(ctx->login.data.passwd) >= NAME_LEN - 1)
 	    len = NAME_LEN - strlen(ctx->login.data.passwd) - 2;
     }
+#ifndef XPM
     if (len == 0)
+#else
+    if (len == 0 || pixels >= LOGIN_W(ctx) - PROMPT_W(ctx))
+#endif /* XPM */
 	return;
     XorCursor (ctx);
     RemoveFail (ctx);
@@ -780,22 +995,45 @@ InsertChar (ctxw, event, params, num_params)
 	       strlen (ctx->login.data.passwd + ctx->login.cursor) + 1);
 	memmove( ctx->login.data.passwd + ctx->login.cursor, strbuf, len);
 	ctx->login.cursor += len;
+
+#ifdef XPM
+	/*as good a place as any Caolan begin*/
+	ctx->login.lastEventTime = time(NULL);
+	/*as good a place as any Caolan end*/
+#endif /* XPM */
 	break;
     }
     XorCursor (ctx);
 }
 
 /* ARGSUSED */
-static void Initialize (greq, gnew, args, num_args)
-    Widget greq, gnew;
-    ArgList args;
-    Cardinal *num_args;
+static void Initialize (
+    Widget greq,
+    Widget gnew,
+    ArgList args,
+    Cardinal *num_args)
 {
     LoginWidget w = (LoginWidget)gnew;
     XtGCMask	valuemask, xvaluemask;
     XGCValues	myXGCV;
     Arg		position[2];
     Position	x, y;
+#ifdef USE_XINERAMA
+    XineramaScreenInfo *screens;
+    int                 s_num;
+#endif
+
+#ifdef XPM
+    myXGCV.foreground = w->login.hipixel;
+    myXGCV.background = w->core.background_pixel;
+    valuemask = GCForeground | GCBackground;
+    w->login.hiGC = XtGetGC(gnew, valuemask, &myXGCV);
+
+    myXGCV.foreground = w->login.shdpixel;
+    myXGCV.background = w->core.background_pixel;
+    valuemask = GCForeground | GCBackground;
+    w->login.shdGC = XtGetGC(gnew, valuemask, &myXGCV);
+#endif /* XPM */
 
     myXGCV.foreground = w->login.textpixel;
     myXGCV.background = w->core.background_pixel;
@@ -852,6 +1090,55 @@ static void Initialize (greq, gnew, args, num_args)
     myXGCV.font = w->login.failFont->fid;
     w->login.failGC = XtGetGC (gnew, xvaluemask, &myXGCV);
 
+#ifdef XPM
+    w->login.logoValid = False;
+
+    if (NULL != w->login.logoFileName)
+    {
+        XpmAttributes myAttributes = { 0 };
+        Window tmpWindow = { 0 };
+        struct stat myBuffer = { 0 };
+        unsigned int myPixmapDepth = 0;
+
+        if (0 != stat(w->login.logoFileName, &myBuffer))
+        {
+            LogError("Unable to stat() pixmap file %s\n",
+                w->login.logoFileName);
+            w->login.logoValid = False;
+            goto SkipXpmLoad;
+        }
+        else
+
+        myAttributes.valuemask |= XpmReturnPixels;
+        myAttributes.valuemask |= XpmReturnExtensions;
+
+        XpmReadFileToPixmap(XtDisplay(w),            /* display */
+            RootWindowOfScreen(XtScreen(w)),         /* window */
+            w->login.logoFileName,                   /* XPM filename */
+            &(w->login.logoPixmap),                  /* pixmap */
+            &(w->login.logoMask),                    /* pixmap mask */
+            &myAttributes);                          /* XPM attributes */
+        w->login.logoValid = True;
+
+        XGetGeometry(XtDisplay(w), w->login.logoPixmap,
+            &tmpWindow,
+            &(w->login.logoX),
+            &(w->login.logoY),
+            &(w->login.logoWidth),
+            &(w->login.logoHeight),
+            &(w->login.logoBorderWidth),
+            &myPixmapDepth);
+    } else {
+	w->login.logoX = 0;
+	w->login.logoY = 0;
+	w->login.logoWidth = 0;
+	w->login.logoHeight = 0;
+	w->login.logoBorderWidth = 0;
+    }
+
+
+SkipXpmLoad:
+#endif /* XPM */
     w->login.data.name[0] = '\0';
     w->login.data.passwd[0] = '\0';
     w->login.state = GET_NAME;
@@ -863,30 +1150,107 @@ static void Initialize (greq, gnew, args, num_args)
 	int fy = FAIL_Y(w);
 	int pady = PAD_Y(w);
 
+#ifndef XPM
 	w->core.height = fy + pady;	/* for stupid compilers */
+#else
+/*	w->core.height = fy + pady;	* for stupid compilers */
+
+        w->core.height = max(fy + pady,
+            (w->login.logoHeight + (2*w->login.logoPadding)) + pady);
+        
+#endif /* XPM */
+    }
+#ifdef USE_XINERAMA
+    if (
+	XineramaIsActive(XtDisplay(w)) &&
+	(screens = XineramaQueryScreens(XtDisplay(w), &s_num)) != NULL
+       )
+    {
+	if ((x = w->core.x) == -1)
+	    x = screens[0].x_org + (int)(screens[0].width - w->core.width) / 2;
+	if ((y = w->core.y) == -1)
+	    y = screens[0].y_org + (int)(screens[0].height - w->core.height) / 3;
+	
+	XFree(screens);
+    }
+    else
+#endif
+    {
+	if ((x = w->core.x) == -1)
+	    x = (int)(XWidthOfScreen (XtScreen (w)) - w->core.width) / 2;
+	if ((y = w->core.y) == -1)
+	    y = (int)(XHeightOfScreen (XtScreen (w)) - w->core.height) / 3;
     }
-    if ((x = w->core.x) == -1)
-	x = (int)(XWidthOfScreen (XtScreen (w)) - w->core.width) / 2;
-    if ((y = w->core.y) == -1)
-	y = (int)(XHeightOfScreen (XtScreen (w)) - w->core.height) / 3;
     XtSetArg (position[0], XtNx, x);
     XtSetArg (position[1], XtNy, y);
     XtSetValues (XtParent (w), position, (Cardinal) 2);
 }
 
  
-static void Realize (gw, valueMask, attrs)
-     Widget gw;
-     XtValueMask *valueMask;
-     XSetWindowAttributes *attrs;
+static void Realize (
+     Widget gw,
+     XtValueMask *valueMask,
+     XSetWindowAttributes *attrs)
 {
+#ifdef XPM
+    LoginWidget	w = (LoginWidget) gw;
+    Cursor cursor;
+#endif /* XPM */
     XtCreateWindow( gw, (unsigned)InputOutput, (Visual *)CopyFromParent,
 		     *valueMask, attrs );
     InitI18N(gw);
+#ifdef XPM
+    cursor = XCreateFontCursor(XtDisplay(gw), XC_left_ptr);
+    XDefineCursor(XtDisplay(gw), XtWindow(gw), cursor);
+
+    /* 
+     * Check if Pixmap was valid
+     */
+    if (True == w->login.logoValid)
+    {
+        /*
+         * Create pixmap window 
+         */
+        {
+            unsigned long valueMask = CWBackPixel | CWBackPixmap;
+            XSetWindowAttributes windowAttributes = { 0 };
+
+            windowAttributes.background_pixel = w->core.background_pixel;
+            windowAttributes.background_pixmap = None;
+
+            w->login.logoWindow = XCreateWindow(XtDisplay(w),
+                XtWindow(w),
+                w->core.width - w->login.outframewidth -
+                    w->login.logoWidth - w->login.logoPadding,
+                (w->core.height - w->login.logoHeight) /2,
+                w->login.logoWidth, w->login.logoHeight, 0,
+                CopyFromParent, InputOutput, CopyFromParent,
+                valueMask, &windowAttributes);
+        }
+
+        /*
+         * check if we can use shape extension
+         */
+        if (True == w->login.useShape)
+        {
+            int foo, bar;
+
+            if (XShapeQueryExtension(XtDisplay(w), &foo, &bar) == TRUE)
+            {
+                XShapeCombineMask(XtDisplay(w), w->login.logoWindow,
+                    ShapeBounding, w->login.logoX, w->login.logoY,
+                    w->login.logoMask, ShapeSet);
+            }
+        }
+
+        XSetWindowBackgroundPixmap(XtDisplay(w), w->login.logoWindow,
+            w->login.logoPixmap);
+        XMapWindow(XtDisplay(w), w->login.logoWindow);
+    }
+#endif /* XPM */
 }
 
-static void Destroy (gw)
-     Widget gw;
+static void Destroy (Widget gw)
 {
     LoginWidget w = (LoginWidget)gw;
     bzero (w->login.data.name, NAME_LEN);
@@ -897,22 +1261,37 @@ static void Destroy (gw)
     XtReleaseGC(gw, w->login.promptGC);
     XtReleaseGC(gw, w->login.greetGC);
     XtReleaseGC(gw, w->login.failGC);
+#ifdef XPM
+    XtReleaseGC(gw, w->login.hiGC);
+    XtReleaseGC(gw, w->login.shdGC);
+
+    if (True == w->login.logoValid)
+    {
+        if (w->login.logoPixmap != 0)
+            XFreePixmap(XtDisplay(w), w->login.logoPixmap);
+
+        if (w->login.logoMask != 0)
+            XFreePixmap(XtDisplay(w), w->login.logoMask);
+    }
+#endif /* XPM */
 }
 
 /* ARGSUSED */
-static void Redisplay(gw, event, region)
-     Widget gw;
-     XEvent *event;
-     Region region;
+static void Redisplay(
+     Widget gw,
+     XEvent *event,
+     Region region)
 {
     draw_it ((LoginWidget) gw);
 }
 
 /*ARGSUSED*/
-static Boolean SetValues (current, request, new, args, num_args)
-    Widget  current, request, new;
-    ArgList args;
-    Cardinal *num_args;
+static Boolean SetValues (
+    Widget  current,
+    Widget  request,
+    Widget  new,
+    ArgList args,
+    Cardinal *num_args)
 {
     LoginWidget currentL, newL;
     
@@ -924,23 +1303,32 @@ static Boolean SetValues (current, request, new, args, num_args)
 }
 
 char defaultLoginTranslations [] =
-"\
-Ctrl<Key>H:	delete-previous-character() \n\
-Ctrl<Key>D:	delete-character() \n\
-Ctrl<Key>B:	move-backward-character() \n\
-Ctrl<Key>F:	move-forward-character() \n\
-Ctrl<Key>A:	move-to-begining() \n\
-Ctrl<Key>E:	move-to-end() \n\
-Ctrl<Key>K:	erase-to-end-of-line() \n\
-Ctrl<Key>U:	erase-line() \n\
-Ctrl<Key>X:	erase-line() \n\
-Ctrl<Key>C:	restart-session() \n\
-Ctrl<Key>\\\\:	abort-session() \n\
-:Ctrl<Key>plus:	allow-all-access() \n\
-<Key>BackSpace:	delete-previous-character() \n\
-<Key>Return:	finish-field() \n\
-<KeyPress>:		insert-char() \
-";
+"Ctrl<Key>H:	delete-previous-character() \n"
+"Ctrl<Key>D:	delete-character() \n"
+"Ctrl<Key>B:	move-backward-character() \n"
+"Ctrl<Key>F:	move-forward-character() \n"
+"Ctrl<Key>A:	move-to-begining() \n"
+"Ctrl<Key>E:	move-to-end() \n"
+"Ctrl<Key>K:	erase-to-end-of-line() \n"
+"Ctrl<Key>U:	erase-line() \n"
+"Ctrl<Key>X:	erase-line() \n"
+"Ctrl<Key>C:	restart-session() \n"
+"Ctrl<Key>\\\\:	abort-session() \n"
+":Ctrl<Key>plus:	allow-all-access() \n"
+"<Key>BackSpace:	delete-previous-character() \n"
+#ifdef linux
+"<Key>Delete:	delete-character() \n"
+#else
+"<Key>Delete:	delete-previous-character() \n"
+#endif
+"<Key>Return:	finish-field() \n"
+#ifndef XPM
+"<KeyPress>:	insert-char()"
+#else
+"<Key>Tab:	tab-field() \n"
+"<KeyPress>:	insert-char()"
+#endif /* XPM */
+;
 
 XtActionsRec loginActionsTable [] = {
   {"delete-previous-character",	DeleteBackwardChar},
@@ -952,6 +1340,9 @@ XtActionsRec loginActionsTable [] = {
   {"erase-to-end-of-line",	EraseToEndOfLine},
   {"erase-line",		EraseLine},
   {"finish-field", 		FinishField},
+#ifdef XPM
+  {"tab-field", 		TabField},
+#endif /* XPM */
   {"abort-session",		AbortSession},
   {"abort-display",		AbortDisplay},
   {"restart-session",		RestartSession},
@@ -985,7 +1376,11 @@ LoginClassRec loginClassRec = {
     /* expose			*/	Redisplay,
     /* set_values		*/	SetValues,
     /* set_values_hook		*/	NULL,
+#ifndef XPM
     /* set_values_almost	*/	NULL,
+#else
+    /* set_values_almost	*/	XtInheritSetValuesAlmost,
+#endif /* XPM */
     /* get_values_hook		*/	NULL,
     /* accept_focus		*/	NULL,
     /* version			*/	XtVersion,
