@@ -1,15 +1,9 @@
 /*
- * $XConsortium: fmalloc.c /main/7 1996/11/24 17:42:06 rws $
- * $XFree86: xc/util/memleak/fmalloc.c,v 3.3.2.1 1999/12/20 14:36:30 hohndel Exp $
+ * $Xorg: fmalloc.c,v 1.4 2000/08/17 19:55:20 cpqbld Exp $
  *
-Copyright (c) 1992  X Consortium
+Copyright 1992, 1998  The Open Group
 
-Permission is hereby granted, free of charge, to any person obtaining a copy
-of this software and associated documentation files (the "Software"), to deal
-in the Software without restriction, including without limitation the rights
-to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-copies of the Software, and to permit persons to whom the Software is
-furnished to do so, subject to the following conditions:
+All Rights Reserved.
 
 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.
@@ -17,17 +11,19 @@ all copies or substantial portions of the Software.
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
-X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
+OPEN GROUP  BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
 AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
-Except as contained in this notice, the name of the X Consortium shall not be
+Except as contained in this notice, the name of The Open Group shall not be
 used in advertising or otherwise to promote the sale, use or other dealings
-in this Software without prior written authorization from the X Consortium.
+in this Software without prior written authorization from The Open Group.
  *
  * Author:  Keith Packard, MIT X Consortium
  */
 
+/* $XFree86: xc/util/memleak/fmalloc.c,v 3.10 2000/12/08 21:43:22 keithp Exp $ */
+
 
 /*
  * Leak tracing allocator -- using C lib malloc/free, tracks
@@ -41,8 +37,15 @@ in this Software without prior written authorization from the X Consortium.
 #include    <stdio.h>
 
 extern char **environ;
+extern xf86DriverList;
 extern etext;
+extern _etext;
 extern __data_start;
+extern _end;
+
+#ifndef TOP_OF_DATA
+#define TOP_OF_DATA 0
+#endif
 
 #ifndef FALSE
 #define FALSE 0
@@ -56,6 +59,7 @@ extern __data_start;
 #endif
 
 typedef unsigned long mem;
+typedef unsigned int  magic;
 
 #ifdef HAS_GET_RETURN_ADDRESS
 #define MAX_RETURN_STACK    16
@@ -87,20 +91,19 @@ typedef struct _head {
     mem		    returnStack[MAX_RETURN_STACK];
 #endif
     mem		    *from;
+    mem             *fromReturnStack;
     unsigned long   allocTime;
     unsigned long   freeTime;
     int		    size;
     int		    desiredsize;
     int		    actualSize;
     int		    marked;
-    int		    headMagic;
+    magic    	    headMagic;
 } HeadRec, *HeadPtr;
 
 typedef struct _tail {
-    int		    tailMagic;
-#if defined(__alpha) || defined(__alpha__)
-    int		    tailPad;
-#endif
+    magic    	    tailMagic;
+    magic    	    tailPad;
 } TailRec, *TailPtr;
 
 #define Header(p)	((HeadPtr) (((char *) (p)) - sizeof (HeadRec)))
@@ -116,7 +119,7 @@ typedef struct _tail {
 typedef HeadRec		tree;
 typedef mem		*tree_data;
 
-#define COMPARE_ADDR(a,b,op)	(((unsigned) (a)) op ((unsigned) (b)))
+#define COMPARE_ADDR(a,b,op)	(((mem) (a)) op ((mem) (b)))
 #define COMPARE(a,b,op,s)	((!s) ? \
 			 COMPARE_ADDR(a,b,op) :\
 			(((a)->actualSize op (b)->actualSize) || \
@@ -139,7 +142,7 @@ typedef mem		*tree_data;
 
 static tree	*activeMemory, *freedMemory, *deadMemory;
 
-static mem	*endOfStaticMemory;
+static mem	*endOfStaticMemory = (mem *) TOP_OF_DATA;
 static mem	*highestAllocatedMemory;
 
 static int	freedMemoryTotal;
@@ -153,6 +156,7 @@ unsigned long	FindLeakAllocBreakpoint = ~0;
 unsigned long	FindLeakFreeBreakpoint = ~0;
 unsigned long	FindLeakTime;
 int		FindLeakCheckAlways = 0;
+int		FindLeakValidateAlways = 0;
 
 static void MarkActiveBlock ();
 static int  tree_insert (), tree_delete ();
@@ -187,10 +191,16 @@ MemError (s, h, ourRet)
 
     if (h)
     {
-	fprintf (stderr, "%s 0x%08lx (size %d) (from 0x%lx)\n",
+	fprintf (stderr, "%s 0x%08lx (size %d) (from 0x%lx) ",
 	     s, DataForHead(h), h->desiredsize, h->from);
 #ifdef HAS_GET_RETURN_ADDRESS
+	if (h->fromReturnStack)
+	    PrintReturnStack ("\nallocated at", h->fromReturnStack);
+	else
+	  fprintf(stderr,"\n");
 	PrintReturnStack ("Saved return stack", h->returnStack);
+#else
+	fprintf(stderr,"\n");
 #endif
     }
     else 
@@ -225,7 +235,7 @@ static void
 MarkActiveBlock (p, from)
     mem   *p, *from;
 {
-    HeadPtr	h;
+    HeadPtr	h, hh;
     int		marked;
     int		oldMarked;
 
@@ -239,6 +249,11 @@ MarkActiveBlock (p, from)
 	{
 	    h->marked |= marked;
 	    h->from = from;
+#ifdef HAS_GET_RETURN_ADDRESS
+	    SEARCH(activeMemory, hh, h->from)
+	      if (hh) 
+		h->fromReturnStack = hh->returnStack;
+#endif
 	    if (!oldMarked)
 		MarkMemoryRegion (DataForHead(h), (mem *) TailForHead(h));
 	}
@@ -254,6 +269,11 @@ MarkActiveBlock (p, from)
 	{
 	    h->marked |= marked;
 	    h->from = from;
+#ifdef HAS_GET_RETURN_ADDRESS
+	    SEARCH(activeMemory, hh, h->from)
+            if (hh) 
+		h->fromReturnStack = hh->returnStack;
+#endif
 	}
 	return;
     }
@@ -355,7 +375,7 @@ ValidateTree (head, headMagic, tailMagic, bodyMagic, mesg)
     char    *mesg;
 {
     TailPtr	tail;
-    mem		*p;
+    magic    	*p;
     int		i;
 
     if (!head)
@@ -367,11 +387,14 @@ ValidateTree (head, headMagic, tailMagic, bodyMagic, mesg)
     if (tail->tailMagic != tailMagic)
 	MemError (mesg, head, FALSE);
     if (bodyMagic) {
-	i = head->size / sizeof (mem);
-	p = DataForHead(head);
+	i = head->size / sizeof (magic);
+	p = (magic *) DataForHead(head);
 	while (i--) {
 	    if (*p++ != bodyMagic)
+	    {
 		MemError (mesg, head, FALSE);
+		break;
+	    }
 	}
     }
     ValidateTree (head->right, headMagic, tailMagic, bodyMagic, mesg);
@@ -396,7 +419,7 @@ AddActiveBlock (h)
     HeadPtr	h;
 {
     TailPtr	t = TailForHead(h);
-    mem		*p;
+    magic    	*p;
     int		i;
 
     tree_insert (&activeMemory, h, FALSE);
@@ -414,8 +437,8 @@ AddActiveBlock (h)
 
     h->headMagic = ACTIVE_HEAD_MAGIC;
     t->tailMagic = ACTIVE_TAIL_MAGIC;
-    i = h->size / sizeof (mem);
-    p = DataForHead(h);
+    i = h->size / sizeof (magic);
+    p = (magic *) DataForHead(h);
     while (i--)
 	*p++ = ACTIVE_DATA_MAGIC;
     activeMemoryTotal += h->desiredsize;
@@ -437,7 +460,7 @@ AddFreedBlock (h)
 {
     TailPtr	t = TailForHead(h);
     int		i;
-    mem		*p;
+    magic    	*p;
 
     tree_insert (&freedMemory, h, FALSE);
 
@@ -452,8 +475,8 @@ AddFreedBlock (h)
 
     h->headMagic = FREED_HEAD_MAGIC;
     t->tailMagic = FREED_TAIL_MAGIC;
-    i = h->size / sizeof (mem);
-    p = DataForHead(h);
+    i = h->size / sizeof (magic);
+    p = (magic *) DataForHead(h);
     while (i--)
 	*p++ = FREED_DATA_MAGIC;
     freedMemoryTotal += h->desiredsize;
@@ -462,7 +485,49 @@ AddFreedBlock (h)
     if (freedMemoryTotal - deadMemoryTotal >= MAX_FREED_MEMORY)
 	CheckMemory ();
 }
+#if 0
+static void
+WarnReferencedRange(rangeStart,rangeEnd,from,to)
+     mem *rangeStart;
+     mem *rangeEnd;
+     mem *from;
+     mem *to;
+{
+  mem *range = rangeStart;
 
+  while ( range < rangeEnd) {
+    if ((mem *)*range >= from && (mem *)*range <= to)
+      fprintf(stderr, "0x%lx still points into newly allocated range\n",
+	      (unsigned long) range);
+    range++;
+  }
+}
+
+static void
+WarnReferencedTree(head, from, to)
+     tree *head;
+     char *from;
+     char *to;
+{
+  if (!head) return;
+  WarnReferencedTree(head->right,from,to);
+  WarnReferencedRange(DataForHead(head),TailForHead(head),from,to);
+  WarnReferencedTree(head->left,from,to);
+}
+
+static void
+WarnReferenced(from, to)
+     char *from;
+     char *to;
+{
+    mem	foo;
+
+    foo = 1;
+    WarnReferencedTree(activeMemory,from,to);
+    WarnReferencedRange(BOTTOM_OF_DATA, endOfStaticMemory,from,to);
+    WarnReferencedRange(&foo, TOP_OF_STACK,from,to);
+}  
+#endif
 /*
  * Entry points:
  *
@@ -478,7 +543,10 @@ CheckMemory ()
 {
     mem	foo;
 
+    foo = 1;
     fprintf (stderr, "\nCheckMemory\n");
+    fprintf (stderr, "Static Memory Area: 0x%lx to 0x%lx\n",
+               BOTTOM_OF_DATA, endOfStaticMemory);
     fprintf (stderr, "%d bytes active memory in %d allocations\n",
 	     activeMemoryTotal, activeMemoryCount);
     fprintf (stderr, "%d bytes freed memory held from %d allocations\n",
@@ -518,7 +586,7 @@ morecore (size)
     {
 	alloc_size = (size + CORE_CHUNK - 1) & ~(CORE_CHUNK-1);
     	newcore = sbrk (alloc_size);
-    	if (((int) newcore) == -1)
+    	if (((long) newcore) == -1)
 	    return 0;
 	core = newcore;
 	core_left = alloc_size;
@@ -543,6 +611,11 @@ malloc (desiredsize)
 	endOfStaticMemory = (mem *) sbrk(0);
     if (FindLeakCheckAlways)
 	CheckMemory ();
+    else if (FindLeakValidateAlways)
+    {
+	ValidateActiveMemory ();
+	ValidateFreedMemory ();
+    }
     size = RoundUp(desiredsize);
     totalsize = TotalSize (size);
 
@@ -624,6 +697,11 @@ free (p)
     AddFreedBlock (h);
     if (FindLeakCheckAlways)
 	CheckMemory ();
+    else if (FindLeakValidateAlways)
+    {
+	ValidateActiveMemory ();
+	ValidateFreedMemory ();
+    }
 }
 
 char *
@@ -638,6 +716,8 @@ realloc (old, desiredsize)
     new = malloc (desiredsize);
     if (!new)
 	return NULL;
+    if (!old)
+	return new;
     SEARCH(activeMemory, h, old);
     if (!h)
     {
